@@ -29,7 +29,7 @@
  */
 
 /*
- * file simulation_input_xml.cpp
+ * file simulation_input_xml.c
  * this file reads the model input from Model_init.xml
  * file using the Expat XML parser.
  * basically a structure with maps is created (see omc_ModelInput)
@@ -176,6 +176,7 @@ typedef struct omc_ModelInput
   omc_ModelVariables    *rAlg; /* algebraic */
   omc_ModelVariables    *rPar; /* parameters */
   omc_ModelVariables    *rAli; /* aliases */
+  omc_ModelVariables    *rSen; /* sensitivities */
 
   omc_ModelVariables    *iAlg; /* int algebraic */
   omc_ModelVariables    *iPar; /* int parameters */
@@ -273,6 +274,8 @@ static void XMLCALL startElement(void *userData, const char *name, const char **
           mi->lastCT = &mi->rPar;
         } else if (0 == strcmp(ct+1,"Ali")) {
           mi->lastCT = &mi->rAli;
+        } else if (0 == strcmp(ct+1,"Sen")) {
+          mi->lastCT = &mi->rSen;
         } else {
           fail = 1;
         }
@@ -314,7 +317,7 @@ static void XMLCALL startElement(void *userData, const char *name, const char **
     }
 
     if (fail) {
-      throwStreamPrint(NULL, "simulation_input_xml.cpp: error reading the xml file, found unknown class: %s  for variable: %s",ct,findHashStringString(v,"name"));
+      throwStreamPrint(NULL, "simulation_input_xml.c: error reading the xml file, found unknown class: %s  for variable: %s",ct,findHashStringString(v,"name"));
     }
 
     /* add the ScalarVariable map to the correct map! */
@@ -371,7 +374,6 @@ static void read_var_info(omc_ScalarVariable *v, VAR_INFO *info)
 
 static void read_var_attribute_real(omc_ScalarVariable *v, REAL_ATTRIBUTE *attribute)
 {
-  read_value_bool(findHashStringString(v,"useStart"), (modelica_boolean*)&(attribute->useStart));
   read_value_real(findHashStringStringEmpty(v,"start"), &(attribute->start), 0.0);
   read_value_bool(findHashStringString(v,"fixed"), (modelica_boolean*)&(attribute->fixed));
   read_value_bool(findHashStringString(v,"useNominal"), (modelica_boolean*)&(attribute->useNominal));
@@ -379,37 +381,34 @@ static void read_var_attribute_real(omc_ScalarVariable *v, REAL_ATTRIBUTE *attri
   read_value_real(findHashStringStringEmpty(v,"min"), &(attribute->min), REAL_MIN);
   read_value_real(findHashStringStringEmpty(v,"max"), &(attribute->max), REAL_MAX);
 
-  infoStreamPrint(LOG_DEBUG, 0, "Real %s(%sstart=%g%s, fixed=%s, %snominal=%g%s, min=%g, max=%g)", findHashStringString(v,"name"), (attribute->useStart)?"":"{", attribute->start, (attribute->useStart)?"":"}", (attribute->fixed)?"true":"false", (attribute->useNominal)?"":"{", attribute->nominal, attribute->useNominal?"":"}", attribute->min, attribute->max);
+  infoStreamPrint(LOG_DEBUG, 0, "Real %s(start=%g, fixed=%s, %snominal=%g%s, min=%g, max=%g)", findHashStringString(v,"name"), attribute->start, (attribute->fixed)?"true":"false", (attribute->useNominal)?"":"{", attribute->nominal, attribute->useNominal?"":"}", attribute->min, attribute->max);
 }
 
 static void read_var_attribute_int(omc_ScalarVariable *v, INTEGER_ATTRIBUTE *attribute)
 {
-  read_value_bool(findHashStringString(v,"useStart"), &attribute->useStart);
   read_value_long(findHashStringStringEmpty(v,"start"), &attribute->start, 0);
   read_value_bool(findHashStringString(v,"fixed"), &attribute->fixed);
   read_value_long(findHashStringStringEmpty(v,"min"), &attribute->min, INTEGER_MIN);
   read_value_long(findHashStringStringEmpty(v,"max"), &attribute->max, INTEGER_MAX);
 
-  infoStreamPrint(LOG_DEBUG, 0, "Integer %s(%sstart=%ld%s, fixed=%s, min=%ld, max=%ld)", findHashStringString(v,"name"), attribute->useStart?"":"{", attribute->start, attribute->useStart?"":"}", attribute->fixed?"true":"false", attribute->min, attribute->max);
+  infoStreamPrint(LOG_DEBUG, 0, "Integer %s(start=%ld, fixed=%s, min=%ld, max=%ld)", findHashStringString(v,"name"), attribute->start, attribute->fixed?"true":"false", attribute->min, attribute->max);
 }
 
 static void read_var_attribute_bool(omc_ScalarVariable *v, BOOLEAN_ATTRIBUTE *attribute)
 {
-  read_value_bool(findHashStringString(v,"useStart"), &attribute->useStart);
   read_value_bool(findHashStringStringEmpty(v,"start"), &attribute->start);
   read_value_bool(findHashStringString(v,"fixed"), &attribute->fixed);
 
-  infoStreamPrint(LOG_DEBUG, 0, "Boolean %s(%sstart=%s%s, fixed=%s)", findHashStringString(v,"name"), attribute->useStart?"":"{", attribute->start?"true":"false", attribute->useStart?"":"}", attribute->fixed?"true":"false");
+  infoStreamPrint(LOG_DEBUG, 0, "Boolean %s(start=%s, fixed=%s)", findHashStringString(v,"name"), attribute->start?"true":"false", attribute->fixed?"true":"false");
 }
 
 static void read_var_attribute_string(omc_ScalarVariable *v, STRING_ATTRIBUTE *attribute)
 {
   const char *start = NULL;
-  read_value_bool(findHashStringString(v,"useStart"), &attribute->useStart);
   read_value_string(findHashStringStringEmpty(v,"start"), &start);
   attribute->start = mmc_mk_scon_persist(start);
 
-  infoStreamPrint(LOG_DEBUG, 0, "String %s(%sstart=%s%s)", findHashStringString(v,"name"), attribute->useStart?"":"{", attribute->start, attribute->useStart?"":"}");
+  infoStreamPrint(LOG_DEBUG, 0, "String %s(start=%s)", findHashStringString(v,"name"), MMC_STRINGDATA(attribute->start));
 }
 
 /* \brief
@@ -425,10 +424,11 @@ void read_input_xml(MODEL_DATA* modelData,
   const char *filename, *guid, *override, *overrideFile;
   FILE* file = NULL;
   XML_Parser parser = NULL;
-  hash_string_long *mapAlias = NULL, *mapAliasParam = NULL;
+  hash_string_long *mapAlias = NULL, *mapAliasParam = NULL, *mapAliasSen = NULL;
   long *it, *itParam;
   mmc_sint_t i;
   int inputIndex = 0;
+  int k = 0;
 
   modelica_integer nxchk, nychk, npchk;
   modelica_integer nyintchk, npintchk;
@@ -444,14 +444,14 @@ void read_input_xml(MODEL_DATA* modelData,
       /* no file given on the command line? use the default
        * model_name defined in generated code for model.*/
       if (0 > GC_asprintf((char**)&filename, "%s_init.xml", modelData->modelFilePrefix)) {
-        throwStreamPrint(NULL, "simulation_input_xml.cpp: Error: can not allocate memory.");
+        throwStreamPrint(NULL, "simulation_input_xml.c: Error: can not allocate memory.");
       }
     }
 
     /* open the file and fail on error. we open it read-write to be sure other processes can overwrite it */
     file = fopen(filename, "r");
     if(!file) {
-      throwStreamPrint(NULL, "simulation_input_xml.cpp: Error: can not read file %s as setup file to the generated simulation code.",filename);
+      throwStreamPrint(NULL, "simulation_input_xml.c: Error: can not read file %s as setup file to the generated simulation code.",filename);
     }
   }
   /* create the XML parser */
@@ -459,7 +459,7 @@ void read_input_xml(MODEL_DATA* modelData,
   if(!parser)
   {
     fclose(file);
-    throwStreamPrint(NULL, "simulation_input_xml.cpp: Error: couldn't allocate memory for the XML parser!");
+    throwStreamPrint(NULL, "simulation_input_xml.c: Error: couldn't allocate memory for the XML parser!");
   }
   /* set our user data */
   XML_SetUserData(parser, &mi);
@@ -476,7 +476,7 @@ void read_input_xml(MODEL_DATA* modelData,
       if(XML_STATUS_ERROR == XML_Parse(parser, buf, len, done))
       {
         fclose(file);
-        warningStreamPrint(LOG_STDOUT, 0, "simulation_input_xml.cpp: Error: failed to read the XML file %s: %s at line %lu\n",
+        warningStreamPrint(LOG_STDOUT, 0, "simulation_input_xml.c: Error: failed to read the XML file %s: %s at line %lu\n",
             filename,
             XML_ErrorString(XML_GetErrorCode(parser)),
             XML_GetCurrentLineNumber(parser));
@@ -487,7 +487,7 @@ void read_input_xml(MODEL_DATA* modelData,
     fclose(file);
   } else if(XML_STATUS_ERROR == XML_Parse(parser, modelData->initXMLData, strlen(modelData->initXMLData), 1)) { /* Got the full string already */
     fprintf(stderr, "%s, %s %lu\n", modelData->initXMLData, XML_ErrorString(XML_GetErrorCode(parser)), XML_GetCurrentLineNumber(parser));
-    warningStreamPrint(LOG_STDOUT, 0, "simulation_input_xml.cpp: Error: failed to read the XML data %s: %s at line %lu\n",
+    warningStreamPrint(LOG_STDOUT, 0, "simulation_input_xml.c: Error: failed to read the XML data %s: %s at line %lu\n",
              modelData->initXMLData,
              XML_ErrorString(XML_GetErrorCode(parser)),
              XML_GetCurrentLineNumber(parser));
@@ -604,18 +604,28 @@ void read_input_xml(MODEL_DATA* modelData,
     { \
       out[j].filterOutput = 1; \
     } \
-    else if (!omc_flag[FLAG_EMIT_PROTECTED] && 0 == strcmp(findHashStringString(v, "isProtected"), "true")) \
+    else if (!omc_flag[FLAG_EMIT_PROTECTED] && 0 == strcmp(findHashStringString(v, "isProtected"), "true") && 0 == strcmp(findHashStringString(v, "hideResult"), "true")) \
     { \
       infoStreamPrint(LOG_DEBUG, 0, "filtering protected variable %s", info->name); \
       out[j].filterOutput = 1; \
     } \
-    else if (!omc_flag[FLAG_IGNORE_HIDERESULT] && 0 == strcmp(findHashStringString(v, "hideResult"), "true")) \
+    else if (!omc_flag[FLAG_IGNORE_HIDERESULT] && 0 == strcmp(findHashStringString(v, "hideResult"), "true") && 0 == strcmp(findHashStringString(v, "isProtected"), "false")) \
     { \
       infoStreamPrint(LOG_DEBUG, 0, "filtering variable %s due to HideResult annotation", info->name); \
       out[j].filterOutput = 1; \
     } \
     addHashStringLong(&mapAlias, info->name, j); /* create a mapping for Alias variable to get the correct index */ \
     debugStreamPrint(LOG_DEBUG, 0, "real %s: mapAlias[%s] = %ld", debugName, info->name, (long) j); \
+    if (omc_flag[FLAG_IDAS] && 0 == strcmp(debugName, "real sensitivities")) \
+    { \
+      if (0 == strcmp(findHashStringString(v, "isValueChangeable"), "true")) \
+      { \
+        long *it = findHashStringLongPtr(mapAliasParam, info->name); \
+        simulationInfo->sensitivityParList[k] = *it; \
+        infoStreamPrint(LOG_SOLVER, 0, "%d. sensitivity parameter %s at index %d", k, info->name, simulationInfo->sensitivityParList[k]); \
+        k++; \
+      } \
+    } \
   } \
   messageClose(LOG_DEBUG);
 
@@ -632,6 +642,11 @@ void read_input_xml(MODEL_DATA* modelData,
   READ_VARIABLES(modelData->booleanParameterData,mi.bPar,BOOLEAN_ATTRIBUTE,read_var_attribute_bool,"boolean parameters",0,modelData->nParametersBoolean,mapAliasParam);
   READ_VARIABLES(modelData->stringParameterData,mi.sPar,STRING_ATTRIBUTE,read_var_attribute_string,"string parameters",0,modelData->nParametersString,mapAliasParam);
 
+  if (omc_flag[FLAG_IDAS])
+  {
+    READ_VARIABLES(modelData->realSensitivityData,mi.rSen,REAL_ATTRIBUTE,read_var_attribute_real,"real sensitivities",0, modelData->nSensitivityVars,mapAliasSen);
+  }
+
   /*
    * real all alias vars
    */
@@ -891,7 +906,7 @@ void doOverride(omc_ModelInput *mi, MODEL_DATA *modelData, const char *override,
   mmc_sint_t i;
   char* overrideStr = NULL;
   if((override != NULL) && (overrideFile != NULL)) {
-    throwStreamPrint(NULL, "simulation_input_xml.cpp: usage error you cannot have both -override and -overrideFile active at the same time. see Model -? for more info!");
+    throwStreamPrint(NULL, "simulation_input_xml.c: usage error you cannot have both -override and -overrideFile active at the same time. see Model -? for more info!");
   }
 
   if(override != NULL) {
@@ -907,9 +922,9 @@ void doOverride(omc_ModelInput *mi, MODEL_DATA *modelData, const char *override,
     /* read override values from file */
     infoStreamPrint(LOG_SOLVER, 0, "read override values from file: %s", overrideFile);
 
-    infile = fopen(overrideFile, "r");
+    infile = fopen(overrideFile, "rb");
     if (0==infile) {
-      throwStreamPrint(NULL, "simulation_input_xml.cpp: could not open the file given to -overrideFile=%s", overrideFile);
+      throwStreamPrint(NULL, "simulation_input_xml.c: could not open the file given to -overrideFile=%s", overrideFile);
     }
 
     free(overrideStr);
@@ -921,7 +936,7 @@ void doOverride(omc_ModelInput *mi, MODEL_DATA *modelData, const char *override,
     errno = 0;
     if (1 != fread(line, n, 1, infile)) {
       free(line);
-      throwStreamPrint(NULL, "simulation_input_xml.cpp: could not read overrideFile %s: %s", overrideFile, strerror(errno));
+      throwStreamPrint(NULL, "simulation_input_xml.c: could not read overrideFile %s: %s", overrideFile, strerror(errno));
     }
     line[n] = '\0';
     overrideLine = (char*) malloc(n+1);
@@ -987,7 +1002,13 @@ void doOverride(omc_ModelInput *mi, MODEL_DATA *modelData, const char *override,
 
     #define CHECK_OVERRIDE(v) \
       if (findHashStringStringNull(mOverrides, findHashStringString(*findHashLongVar(mi->v,i),"name"))) { \
+        if (0 == strcmp(findHashStringString(*findHashLongVar(mi->v,i), "isValueChangeable"), "true")){ \
         addHashStringString(findHashLongVar(mi->v,i), "start", getOverrideValue(mOverrides, &mOverridesUses, findHashStringString(*findHashLongVar(mi->v,i),"name"))); \
+        } \
+        else{ \
+          addHashStringLong(&mOverridesUses, findHashStringString(*findHashLongVar(mi->v,i),"name"), OMC_OVERRIDE_USED); \
+          warningStreamPrint(LOG_STDOUT, 0, "It is not possible to override the following quantity: %s\nIt seems to be structural, final, protected or evaluated or has a non-constant binding.", findHashStringString(*findHashLongVar(mi->v,i),"name")); \
+        } \
       }
 
     // override all found!
@@ -1039,7 +1060,7 @@ void doOverride(omc_ModelInput *mi, MODEL_DATA *modelData, const char *override,
     // give a warning if an override is not used #3204
     HASH_ITER(hh, mOverridesUses, it, ittmp) {
       if (it->val == OMC_OVERRIDE_UNUSED) {
-        warningStreamPrint(LOG_STDOUT, 0, "simulation_input_xml.cpp: override variable name not found in model: %s\n", it->id);
+        warningStreamPrint(LOG_STDOUT, 0, "simulation_input_xml.c: override variable name not found in model: %s\n", it->id);
       }
     }
 
