@@ -107,6 +107,8 @@ typedef struct CALL_STATISTICS
   long updateDiscreteSystem;
   long functionZeroCrossingsEquations;
   long functionZeroCrossings;
+  long functionEvalDAE;
+  long functionAlgebraics;
 } CALL_STATISTICS;
 
 typedef enum {ERROR_AT_TIME,NO_PROGRESS_START_POINT,NO_PROGRESS_FACTOR,IMPROPER_INPUT} equationSystemError;
@@ -170,41 +172,19 @@ typedef struct EXTERNAL_INPUT
 }EXTERNAL_INPUT;
 
 /* Alias data with various types*/
-typedef struct DATA_REAL_ALIAS
+typedef struct DATA_ALIAS
 {
   int negate;
   int nameID;                          /* pointer to Alias */
   char aliasType;                      /* 0 variable, 1 parameter, 2 time */
   VAR_INFO info;
   modelica_boolean filterOutput;       /* true if this variable should be filtered */
-}DATA_REAL_ALIAS;
+} DATA_ALIAS;
 
-typedef struct DATA_INTEGER_ALIAS
-{
-  int negate;
-  int nameID;
-  char aliasType;                      /* 0 variable, 1 parameter */
-  VAR_INFO info;
-  modelica_boolean filterOutput;       /* true if this variable should be filtered */
-}DATA_INTEGER_ALIAS;
-
-typedef struct DATA_BOOLEAN_ALIAS
-{
-  int negate;
-  int nameID;
-  char aliasType;                      /* 0 variable, 1 parameter */
-  VAR_INFO info;
-  modelica_boolean filterOutput;       /* true if this variable should be filtered */
-}DATA_BOOLEAN_ALIAS;
-
-typedef struct DATA_STRING_ALIAS
-{
-  int negate;
-  int nameID;
-  char aliasType;                      /* 0 variable, 1 parameter */
-  VAR_INFO info;
-  modelica_boolean filterOutput;       /* true if this variable should be filtered */
-}DATA_STRING_ALIAS;
+typedef DATA_ALIAS DATA_REAL_ALIAS;
+typedef DATA_ALIAS DATA_INTEGER_ALIAS;
+typedef DATA_ALIAS DATA_BOOLEAN_ALIAS;
+typedef DATA_ALIAS DATA_STRING_ALIAS;
 
 /* collect all attributes from one variable in one struct */
 typedef struct REAL_ATTRIBUTE
@@ -216,7 +196,6 @@ typedef struct REAL_ATTRIBUTE
   modelica_boolean fixed;              /* depends on the type */
   modelica_boolean useNominal;         /* = false */
   modelica_real nominal;               /* = 1.0 */
-  modelica_boolean useStart;           /* = false */
   modelica_real start;                 /* = 0.0 */
 }REAL_ATTRIBUTE;
 
@@ -225,20 +204,17 @@ typedef struct INTEGER_ATTRIBUTE
   modelica_integer min;                /* = -Inf */
   modelica_integer max;                /* = +Inf */
   modelica_boolean fixed;              /* depends on the type */
-  modelica_boolean useStart;           /* = false */
   modelica_integer start;              /* = 0 */
 }INTEGER_ATTRIBUTE;
 
 typedef struct BOOLEAN_ATTRIBUTE
 {
   modelica_boolean fixed;              /* depends on the type */
-  modelica_boolean useStart;           /* = false */
   modelica_boolean start;              /* = false */
 }BOOLEAN_ATTRIBUTE;
 
 typedef struct STRING_ATTRIBUTE
 {
-  modelica_boolean useStart;           /* = false */
   modelica_string start;               /* = "" */
 }STRING_ATTRIBUTE;
 
@@ -270,6 +246,7 @@ typedef struct STATIC_STRING_DATA
   modelica_boolean filterOutput;       /* true if this variable should be filtered */
 }STATIC_STRING_DATA;
 
+#if !defined(OMC_NUM_NONLINEAR_SYSTEMS) || OMC_NUM_NONLINEAR_SYSTEMS>0
 typedef struct NONLINEAR_SYSTEM_DATA
 {
   modelica_integer size;
@@ -295,6 +272,7 @@ typedef struct NONLINEAR_SYSTEM_DATA
   void (*residualFunc)(void**, const double*, double*, const int*);
   void (*initializeStaticNLSData)(void*, threadData_t *threadData, void*);
   int (*strictTearingFunctionCall)(struct DATA*, threadData_t *threadData);
+  void (*getIterationVars)(struct DATA*, double*);
 
   void *solverData;
   modelica_real *nlsx;                 /* x */
@@ -302,10 +280,11 @@ typedef struct NONLINEAR_SYSTEM_DATA
   modelica_real *nlsxExtrapolation;    /* extrapolated values for x from old and old2 - used as initial guess */
 
   void *oldValueList;                  /* old values organized in a sorted list for extrapolation and interpolate, respectively */
+  modelica_real *resValues;            /* memory space for evaluated residual values */
 
-  modelica_integer method;             /* used for linear tearing system if 1: Newton step is done otherwise 0 */
   modelica_real residualError;         /* not used */
   modelica_boolean solved;             /* 1: solved in current step - else not */
+  modelica_real lastTimeSolved;         /* save last successful solved point in time */
 
   /* statistics */
   unsigned long numberOfCall;           /* number of solving calls of this system */
@@ -313,10 +292,13 @@ typedef struct NONLINEAR_SYSTEM_DATA
   unsigned long numberOfIterations;     /* number of iteration of non-linear solvers of this system */
   double totalTime;                     /* save the totalTime */
   rtclock_t totalTimeClock;             /* time clock for the totalTime  */
-
   void* csvData;                        /* information to save csv data */
 } NONLINEAR_SYSTEM_DATA;
+#else
+typedef void* NONLINEAR_SYSTEM_DATA;
+#endif
 
+#if !defined(OMC_NUM_LINEAR_SYSTEMS) || OMC_NUM_LINEAR_SYSTEMS>0
 typedef struct LINEAR_SYSTEM_DATA
 {
   /* set matrix A */
@@ -333,7 +315,7 @@ typedef struct LINEAR_SYSTEM_DATA
 
   void (*residualFunc)(void**, const double*, double*, const int*);
   void (*initializeStaticLSData)(void*, threadData_t *threadData, void*);
-
+  int (*strictTearingFunctionCall)(struct DATA*, threadData_t *threadData);
 
   /* attributes of iteration variables */
   modelica_real *min;
@@ -353,13 +335,18 @@ typedef struct LINEAR_SYSTEM_DATA
   modelica_real residualError;          /* not used yet*/
   modelica_boolean solved;              /* 1: solved in current step - else not */
   modelica_boolean failed;              /* 1: failed while last try with lapack - else not */
+  modelica_boolean useSparseSolver;     /* 1: use sparse solver, - else any solver */
 
   /* statistics */
   unsigned long numberOfCall;           /* number of solving calls of this system */
   double totalTime;                     /* save the totalTime */
   rtclock_t totalTimeClock;             /* time clock for the totalTime  */
 }LINEAR_SYSTEM_DATA;
+#else
+typedef void* LINEAR_SYSTEM_DATA;
+#endif
 
+#if !defined(OMC_NUM_MIXED_SYSTEMS) || OMC_NUM_MIXED_SYSTEMS>0
 typedef struct MIXED_SYSTEM_DATA
 {
   modelica_integer size;
@@ -378,7 +365,11 @@ typedef struct MIXED_SYSTEM_DATA
   modelica_integer method;          /* not used yet*/
   modelica_boolean solved;          /* 1: solved in current step - else not */
 }MIXED_SYSTEM_DATA;
+#else
+typedef void* MIXED_SYSTEM_DATA;
+#endif
 
+#if !defined(OMC_NO_STATESELECTION)
 typedef struct STATE_SET_DATA
 {
   modelica_integer nCandidates;
@@ -403,6 +394,38 @@ typedef struct STATE_SET_DATA
   int (*initialAnalyticalJacobian)(void*, threadData_t*);
   modelica_integer jacobianIndex;
 }STATE_SET_DATA;
+#else
+typedef void* STATE_SET_DATA;
+#endif
+
+typedef struct DAEMODE_DATA
+{
+  /* number of dae residual variables */
+  long nResidualVars;
+
+  /* number of algebraic variables */
+  long nAlgebraicDAEVars;
+
+  /* workspace for the residual variables */
+  modelica_real* residualVars;
+
+  /* daeMode sparse pattern */
+  SPARSE_PATTERN* sparsePattern;
+
+  /* function to evaluate dynamic equations for DAE solver*/
+  int (*evaluateDAEResiduals)(struct DATA*, threadData_t*);
+
+  /* function to set algebraic DAE Variable form solver*/
+  int (*setAlgebraicDAEVars)(struct DATA*, threadData_t*, double*);
+
+  /* function to get algebraic DAE Variable form solver*/
+  int (*getAlgebraicDAEVars)(struct DATA*, threadData_t*, double*);
+
+  /* function to get algebraic DAE nominal values*/
+  int (*getAlgebraicDAEVarNominals)(struct DATA*, threadData_t*, double*);
+
+} DAEMODE_DATA;
+
 
 typedef struct MODEL_DATA_XML
 {
@@ -446,6 +469,8 @@ typedef struct MODEL_DATA
   DATA_BOOLEAN_ALIAS* booleanAlias;
   DATA_STRING_ALIAS* stringAlias;
 
+  STATIC_REAL_DATA* realSensitivityData;
+
   MODEL_DATA_XML modelDataXml;         /* TODO: Rename me? */
 
   const char* modelName;
@@ -495,6 +520,9 @@ typedef struct MODEL_DATA
   long nAliasString;
 
   long nJacobians;
+
+  long nSensitivityVars;
+  long nSensitivityParamVars;
 }MODEL_DATA;
 
 typedef struct CLOCK_DATA {
@@ -519,18 +547,22 @@ typedef struct SIMULATION_INFO
 {
   modelica_real startTime;
   modelica_real stopTime;
+  int useStopTime;
   modelica_integer numSteps;
   modelica_real stepSize;
   modelica_real tolerance;
   const char *solverMethod;
   const char *outputFormat;
   const char *variableFilter;
+
   int lsMethod;                        /* linear solver */
+  int lssMethod;                       /* linear sparse solver */
   int mixedMethod;                     /* mixed solver */
+
   int nlsMethod;                       /* nonlinear solver */
   int newtonStrategy;                  /* newton damping strategy solver */
   int nlsCsvInfomation;                /* = 1 csv files with detailed nonlinear solver process are generated */
-
+  int nlsLinearSolver;                 /* nls linear solver setting =1 totalpivot, =2 lapack, =3=klu */
   /* current context evaluation, set by dassl and used for extrapolation
    * of next non-linear guess */
   int currentContext;
@@ -550,6 +582,8 @@ typedef struct SIMULATION_INFO
   modelica_boolean solveContinuous;    /* =1 during the continuous integration to avoid zero-crossings jumps,  0 otherwise. */
   modelica_boolean noThrowDivZero;     /* =1 if solving nonlinear system to avoid THROW for division by zero,  0 otherwise. */
 
+  double solverSteps;                  /* Number of integration steps so far for writing to the result file*/
+
   void** extObjs;                      /* External objects */
 
   double nextSampleEvent;              /* point in time of next sample-call */
@@ -589,6 +623,9 @@ typedef struct SIMULATION_INFO
   modelica_real* outputVars;
   EXTERNAL_INPUT external_input;
 
+  modelica_real* sensitivityMatrix;    /* used by integrator for sensitivity mode  */
+  int* sensitivityParList;             /* used by integrator for sensitivity mode  */
+
   ANALYTIC_JACOBIAN* analyticJacobians;
 
   NONLINEAR_SYSTEM_DATA* nonlinearSystemData;
@@ -601,6 +638,8 @@ typedef struct SIMULATION_INFO
 
   STATE_SET_DATA* stateSetData;
 
+  DAEMODE_DATA* daeModeData;
+
   /* delay vars */
   double tStart;
   RINGBUFFER **delayStructure;
@@ -624,6 +663,16 @@ typedef struct SIMULATION_DATA
 
 }SIMULATION_DATA;
 
+#if !defined(OMC_MINIMAL_RUNTIME)
+typedef struct {
+  int enabled;
+  double scaling;
+  double time;
+  rtclock_t clock;
+  int64_t maxLate;
+} real_time_sync_t;
+#endif
+
 /* top-level struct to collect dynamic and static model data */
 typedef struct DATA
 {
@@ -632,6 +681,10 @@ typedef struct DATA
   MODEL_DATA *modelData;                /* static stuff */
   SIMULATION_INFO *simulationInfo;
   struct OpenModelicaGeneratedFunctionCallbacks *callback;
+#if !defined(OMC_MINIMAL_RUNTIME)
+  void *embeddedServerState; /* Variable sent around controlling the state of the embedded server */
+  real_time_sync_t real_time_sync;
+#endif
 } DATA;
 
 #include "openmodelica_func.h"
