@@ -28,26 +28,30 @@
  *
  */
 /*
- *
- * RCS: $Id: BreakpointsWidget.cpp 25271 2015-03-26 13:34:40Z adeas31 $
- *
+ * @author Adeel Asghar <adeel.asghar@liu.se>
  */
 
 #include "BreakpointsWidget.h"
+#include "Debugger/GDB/GDBAdapter.h"
 #include "BreakpointDialog.h"
-#include "CommandFactory.h"
+#include "Debugger/GDB/CommandFactory.h"
+#include "Util/Helper.h"
+#include "Modeling/ModelWidgetContainer.h"
+
+#include <QGridLayout>
+#include <QMenu>
 
 /*!
-  \class BreakPointsWidget
-  \brief A widget containing BreakpointsTreeView.
-  */
+ * \class BreakPointsWidget
+ * \brief A widget containing BreakpointsTreeView.
+ */
 /*!
-  \param pMainWindow - pointer to DebuggerMainWindow
-  */
-BreakpointsWidget::BreakpointsWidget(DebuggerMainWindow *pDebuggerMainWindow)
-  : QWidget(pDebuggerMainWindow)
+ * \brief BreakpointsWidget::BreakpointsWidget
+ * \param pParent
+ */
+BreakpointsWidget::BreakpointsWidget(QWidget *pParent)
+  : QWidget(pParent)
 {
-  mpDebuggerMainWindow = pDebuggerMainWindow;
   /* Breakpoints Tree view */
   mpBreakpointsTreeView = new BreakpointsTreeView(this);
   mpBreakpointsTreeModel = new BreakpointsTreeModel(mpBreakpointsTreeView);
@@ -78,6 +82,7 @@ BreakpointsTreeView::BreakpointsTreeView(BreakpointsWidget *pBreakPointsWidget)
   setIconSize(QSize(15, 15));
   setExpandsOnDoubleClick(false);
   setContextMenuPolicy(Qt::CustomContextMenu);
+  setUniformRowHeights(true);
   createActions();
   connect(this, SIGNAL(customContextMenuRequested(QPoint)), SLOT(showContextMenu(QPoint)));
   connect(this, SIGNAL(doubleClicked(QModelIndex)), SLOT(breakPointDoubleClicked(QModelIndex)));
@@ -261,7 +266,7 @@ BreakpointsTreeModel::BreakpointsTreeModel(BreakpointsTreeView *pBreakpointsTree
 {
   mpBreakpointsTreeView = pBreakpointsTreeView;
   QVector<QVariant> headers;
-  headers << tr("File") << Helper::line;
+  headers << Helper::file << Helper::line;
   mpRootBreakpointTreeItem = new BreakpointTreeItem(headers);
   mpRootBreakpointTreeItem->setIsRootItem(true);
 }
@@ -429,9 +434,8 @@ void BreakpointsTreeModel::insertBreakpoint(BreakpointMarker *pBreakpointMarker,
   pParentBreakpointTreeItem->insertChild(row, pBreakpointTreeItem);
   endInsertRows();
   // insert the breakpoint in gdb
-  GDBAdapter *pGDBAdapter = mpBreakpointsTreeView->getBreakpointsWidget()->getDebuggerMainWindow()->getGDBAdapter();
-  if (pGDBAdapter->isGDBRunning()) {
-    pGDBAdapter->insertBreakpoint(pBreakpointTreeItem);
+  if (GDBAdapter::instance()->isGDBRunning()) {
+    GDBAdapter::instance()->insertBreakpoint(pBreakpointTreeItem);
   }
 }
 
@@ -462,26 +466,25 @@ void BreakpointsTreeModel::updateBreakpoint(BreakpointTreeItem *pBreakpointTreeI
                                             int ignoreCount, QString condition)
 {
   // enable/disable the breakpoint in gdb.
-  GDBAdapter *pGDBAdapter = mpBreakpointsTreeView->getBreakpointsWidget()->getDebuggerMainWindow()->getGDBAdapter();
-  if (pGDBAdapter->isGDBRunning() && !pBreakpointTreeItem->getBreakpointID().isEmpty()) {
+  if (GDBAdapter::instance()->isGDBRunning() && !pBreakpointTreeItem->getBreakpointID().isEmpty()) {
     if (pBreakpointTreeItem->isEnabled() != enabled) {
       if (enabled) {
-        pGDBAdapter->postCommand(CommandFactory::breakEnable(QStringList() << pBreakpointTreeItem->getBreakpointID()),
-                                 GDBAdapter::NonCriticalResponse);
+        GDBAdapter::instance()->postCommand(CommandFactory::breakEnable(QStringList() << pBreakpointTreeItem->getBreakpointID()),
+                                            GDBAdapter::NonCriticalResponse);
       } else {
-        pGDBAdapter->postCommand(CommandFactory::breakDisable(QStringList() << pBreakpointTreeItem->getBreakpointID()),
-                                 GDBAdapter::NonCriticalResponse);
+        GDBAdapter::instance()->postCommand(CommandFactory::breakDisable(QStringList() << pBreakpointTreeItem->getBreakpointID()),
+                                            GDBAdapter::NonCriticalResponse);
       }
     }
     // add the ignore count in gdb
     if (pBreakpointTreeItem->getIgnoreCount() != ignoreCount) {
-      pGDBAdapter->postCommand(CommandFactory::breakAfter(pBreakpointTreeItem->getBreakpointID(), ignoreCount),
-                               GDBAdapter::NonCriticalResponse);
+      GDBAdapter::instance()->postCommand(CommandFactory::breakAfter(pBreakpointTreeItem->getBreakpointID(), ignoreCount),
+                                          GDBAdapter::NonCriticalResponse);
     }
     // add the condition in gdb
     if (pBreakpointTreeItem->getCondition().compare(condition) != 0) {
-      pGDBAdapter->postCommand(CommandFactory::breakCondition(pBreakpointTreeItem->getBreakpointID(), condition),
-                               GDBAdapter::NonCriticalResponse);
+      GDBAdapter::instance()->postCommand(CommandFactory::breakCondition(pBreakpointTreeItem->getBreakpointID(), condition),
+                                          GDBAdapter::NonCriticalResponse);
     }
   }
   // update the breakpoint in the tree.
@@ -512,13 +515,11 @@ void BreakpointsTreeModel::removeBreakpoint(BreakpointMarker *pBreakpointMarker)
   */
 void BreakpointsTreeModel::removeBreakpoint(BreakpointTreeItem *pBreakpointTreeItem)
 {
-  if (pBreakpointTreeItem)
-  {
+  if (pBreakpointTreeItem) {
     // remove the breakpoint in gdb
-    GDBAdapter *pGDBAdapter = mpBreakpointsTreeView->getBreakpointsWidget()->getDebuggerMainWindow()->getGDBAdapter();
-    if (pGDBAdapter->isGDBRunning() && !pBreakpointTreeItem->getBreakpointID().isEmpty()) {
-      pGDBAdapter->postCommand(CommandFactory::breakDelete(QStringList() << pBreakpointTreeItem->getBreakpointID()),
-                               GDBAdapter::NonCriticalResponse);
+    if (GDBAdapter::instance()->isGDBRunning() && !pBreakpointTreeItem->getBreakpointID().isEmpty()) {
+      GDBAdapter::instance()->postCommand(CommandFactory::breakDelete(QStringList() << pBreakpointTreeItem->getBreakpointID()),
+                                          GDBAdapter::NonCriticalResponse);
     }
     // remove the breakpoint from the tree.
     int row = pBreakpointTreeItem->row();
