@@ -29,132 +29,132 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- * RCS: $Id$
- *
  */
 
 #ifndef MAINWINDOW_H
 #define MAINWINDOW_H
 
-#include <QtGlobal>
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
-#include <QtWidgets>
-#include <QPrinter>
-#include <QPrintDialog>
-#include <QtWebKitWidgets>
-#include <QTextCodec>
-#include <QUrlQuery>
-#include <QItemDelegate>
-#include <QDomDocument>
-#else
-#include <QtGui>
-#include <QtWebKit>
-#include <QtCore>
-#include <QDomDocument>
-#endif
+#undef smooth
 
+#include <QtGlobal>
 #if (QT_VERSION < QT_VERSION_CHECK(4, 6, 0))
 #error "OMEdit requires Qt 4.6.0 or newer"
 #endif
 
-#include "OMCProxy.h"
-#include "OptionsDialog.h"
-#include "ModelicaClassDialog.h"
-#include "StringHandler.h"
-#include "MessagesWidget.h"
-#include "TransformationsWidget.h"
-#include "LibraryTreeWidget.h"
-#include "DocumentationWidget.h"
-#include "SimulationDialog.h"
-#include "TLMCoSimulationDialog.h"
-#include "Plotting/PlotWindowContainer.h"
-#include "ModelWidgetContainer.h"
-#include "DebuggerMainWindow.h"
-#include "ImportFMUDialog.h"
-#include "NotificationsDialog.h"
+#include <QMainWindow>
+#include <QWidget>
+#include <QProgressBar>
+#include <QDomDocument>
+#include <QStackedWidget>
+#include <QActionGroup>
+#include <QToolButton>
+#include <QMdiSubWindow>
+#include <QMdiArea>
 
 class OMCProxy;
-class OptionsDialog;
-class MessagesWidget;
 class TransformationsWidget;
 class LibraryWidget;
+class GDBAdapter;
+class StackFramesWidget;
+class LocalsWidget;
+class TargetOutputWidget;
+class GDBLoggerWidget;
 class DocumentationWidget;
+class PlotWindowContainer;
 class VariablesWidget;
+class BreakpointsWidget;
 class SimulationDialog;
 class TLMCoSimulationDialog;
-class PlotWindowContainer;
 class ModelWidgetContainer;
-class DebuggerMainWindow;
-class InfoBar;
 class WelcomePageWidget;
+class InfoBar;
 class AboutOMEditWidget;
+class Label;
+class FileDataNotifier;
+class LibraryTreeItem;
 
 class MainWindow : public QMainWindow
 {
   Q_OBJECT
 public:
   enum { MaxRecentFiles = 8 };
-  MainWindow(QSplashScreen *pSplashScreen, bool debug, QWidget *parent = 0);
+private:
+  MainWindow(bool debug, QWidget *parent = 0);
+  static MainWindow *mpInstance;
+public:
+  static MainWindow *instance(bool debug = false);
+  void setUpMainWindow();
   bool isDebug() {return mDebug;}
-  OMCProxy* getOMCProxy();
-  void setExitApplicationStatus(bool status);
-  bool getExitApplicationStatus();
-  OptionsDialog* getOptionsDialog();
-  MessagesWidget* getMessagesWidget();
-  LibraryWidget* getLibraryWidget();
-  DocumentationWidget* getDocumentationWidget();
-  QDockWidget* getDocumentationDockWidget();
-  VariablesWidget* getVariablesWidget();
-  QDockWidget* getVariablesDockWidget();
-  SimulationDialog* getSimulationDialog();
+  OMCProxy* getOMCProxy() {return mpOMCProxy;}
+  void setExitApplicationStatus(bool status) {mExitApplicationStatus = status;}
+  bool getExitApplicationStatus() {return mExitApplicationStatus;}
+  LibraryWidget* getLibraryWidget() {return mpLibraryWidget;}
+  StackFramesWidget* getStackFramesWidget() {return mpStackFramesWidget;}
+  BreakpointsWidget* getBreakpointsWidget() {return mpBreakpointsWidget;}
+  LocalsWidget* getLocalsWidget() {return mpLocalsWidget;}
+  TargetOutputWidget* getTargetOutputWidget() {return mpTargetOutputWidget;}
+  QDockWidget* getTargetOutputDockWidget() {return mpTargetOutputDockWidget;}
+  GDBLoggerWidget* getGDBLoggerWidget() {return mpGDBLoggerWidget;}
+  DocumentationWidget* getDocumentationWidget() {return mpDocumentationWidget;}
+  QDockWidget* getDocumentationDockWidget() {return mpDocumentationDockWidget;}
+  PlotWindowContainer* getPlotWindowContainer() {return mpPlotWindowContainer;}
+  VariablesWidget* getVariablesWidget() {return mpVariablesWidget;}
+  QDockWidget* getVariablesDockWidget() {return mpVariablesDockWidget;}
+  SimulationDialog* getSimulationDialog() {return mpSimulationDialog;}
   TLMCoSimulationDialog* getTLMCoSimulationDialog() {return mpTLMCoSimulationDialog;}
-  PlotWindowContainer* getPlotWindowContainer();
-  ModelWidgetContainer* getModelWidgetContainer();
-  DebuggerMainWindow* getDebuggerMainWindow() {return mpDebuggerMainWindow;}
-  WelcomePageWidget* getWelcomePageWidget();
-  InfoBar* getInfoBar();
-  QStatusBar* getStatusBar();
-  QProgressBar* getProgressBar();
-  Label* getPointerXPositionLabel();
-  Label* getPointerYPositionLabel();
-  QTabBar* getPerspectiveTabBar();
-  QAction* getSaveAction();
-  QAction* getSaveAsAction();
+  ModelWidgetContainer* getModelWidgetContainer() {return mpModelWidgetContainer;}
+  WelcomePageWidget* getWelcomePageWidget() {return mpWelcomePageWidget;}
+  QStatusBar* getStatusBar() {return mpStatusBar;}
+  QProgressBar* getProgressBar() {return mpProgressBar;}
+  void showProgressBar() {mpProgressBar->setVisible(true);}
+  void hideProgressBar() {mpProgressBar->setVisible(false);}
+  Label* getPointerXPositionLabel() {return mpPointerXPositionLabel;}
+  Label* getPointerYPositionLabel() {return mpPointerYPositionLabel;}
+  QTabBar* getPerspectiveTabBar() {return mpPerspectiveTabbar;}
+  QToolBar* getPlotToolBar() {return mpPlotToolBar;}
+  QTimer* getAutoSaveTimer() {return mpAutoSaveTimer;}
+  QAction* getSaveAction() {return mpSaveAction;}
+  QAction* getSaveAsAction() {return mpSaveAsAction;}
   QAction* getSaveTotalAction() {return mpSaveTotalAction;}
-  QAction* getPrintModelAction();
-  QAction* getSaveAllAction();
+  QAction* getPrintModelAction() {return mpPrintModelAction;}
+  QAction* getSaveAllAction() {return mpSaveAllAction;}
   QAction* getUndoAction() {return mpUndoAction;}
   QAction* getRedoAction() {return mpRedoAction;}
-  QAction* getShowGridLinesAction();
-  QAction* getResetZoomAction();
-  QAction* getZoomInAction();
-  QAction* getZoomOutAction();
-  QAction* getSimulateModelAction();
+  QAction* getShowGridLinesAction() {return mpShowGridLinesAction;}
+  QAction* getResetZoomAction() {return mpResetZoomAction;}
+  QAction* getZoomInAction() {return mpZoomInAction;}
+  QAction* getZoomOutAction() {return mpZoomOutAction;}
+  QAction* getSimulateModelAction() {return mpSimulateModelAction;}
   QAction* getSimulateWithTransformationalDebuggerAction() {return mpSimulateWithTransformationalDebuggerAction;}
   QAction* getSimulateWithAlgorithmicDebuggerAction() {return mpSimulateWithAlgorithmicDebuggerAction;}
-  QAction* getSimulationSetupAction();
-  QAction* getInstantiateModelAction();
-  QAction* getCheckModelAction();
+#if !defined(WITHOUT_OSG)
+  QAction* getSimulateWithAnimationAction() {return mpSimulateWithAnimationAction;}
+#endif
+  QAction* getSimulationSetupAction() {return mpSimulationSetupAction;}
+  QAction* getInstantiateModelAction() {return mpInstantiateModelAction;}
+  QAction* getCheckModelAction() {return mpCheckModelAction;}
   QAction* getCheckAllModelsAction() {return mpCheckAllModelsAction;}
-  QAction* getExportFMUAction();
-  QAction* getExportXMLAction();
-  QAction* getExportFigaroAction();
-  QAction* getLineShapeAction();
-  QAction* getPolygonShapeAction();
-  QAction* getRectangleShapeAction();
-  QAction* getEllipseShapeAction();
-  QAction* getTextShapeAction();
-  QAction* getBitmapShapeAction();
-  QAction* getExportAsImageAction();
+  QAction* getExportFMUAction() {return mpExportFMUAction;}
+  QAction* getExportXMLAction() {return mpExportXMLAction;}
+  QAction* getExportFigaroAction() {return mpExportFigaroAction;}
+  QAction* getLineShapeAction() {return mpLineShapeAction;}
+  QAction* getPolygonShapeAction() {return mpPolygonShapeAction;}
+  QAction* getRectangleShapeAction() {return mpRectangleShapeAction;}
+  QAction* getEllipseShapeAction() {return mpEllipseShapeAction;}
+  QAction* getTextShapeAction() {return mpTextShapeAction;}
+  QAction* getBitmapShapeAction() {return mpBitmapShapeAction;}
+  QAction* getExportAsImageAction() {return mpExportAsImageAction;}
   QAction* getExportToClipboardAction() {return mpExportToClipboardAction;}
-  QAction* getExportToOMNotebookAction();
-  QAction* getImportFromOMNotebookAction();
-  QAction* getImportNgspiceNetlistAction();
-  QAction* getConnectModeAction();
+  QAction* getExportToOMNotebookAction() {return mpExportToOMNotebookAction;}
+  QAction* getImportFromOMNotebookAction() {return mpImportFromOMNotebookAction;}
+  QAction* getImportNgspiceNetlistAction() {return mpImportNgspiceNetlistAction;}
+  QAction* getConnectModeAction() {return mpConnectModeAction;}
+  QAction* getReSimulateModelAction() {return mpReSimulateModelAction;}
+  QAction* getReSimulateSetupAction() {return mpReSimulateSetupAction;}
+  QAction* getSimulationParamsAction() {return mpSimulationParamsAction;}
   QAction* getFetchInterfaceDataAction() {return mpFetchInterfaceDataAction;}
+  QAction* getAlignInterfacesAction() {return mpAlignInterfacesAction;}
   QAction* getTLMSimulationAction() {return mpTLMCoSimulationAction;}
   void addRecentFile(const QString &fileName, const QString &encoding);
   void updateRecentFileActions();
@@ -166,6 +166,9 @@ public:
   void simulate(LibraryTreeItem *pLibraryTreeItem);
   void simulateWithTransformationalDebugger(LibraryTreeItem *pLibraryTreeItem);
   void simulateWithAlgorithmicDebugger(LibraryTreeItem *pLibraryTreeItem);
+#if !defined(WITHOUT_OSG)
+  void simulateWithAnimation(LibraryTreeItem *pLibraryTreeItem);
+#endif
   void simulationSetup(LibraryTreeItem *pLibraryTreeItem);
   void instantiateModel(LibraryTreeItem *pLibraryTreeItem);
   void checkModel(LibraryTreeItem *pLibraryTreeItem);
@@ -180,6 +183,7 @@ public:
   void createOMNotebookImageCell(LibraryTreeItem *pLibraryTreeItem, QDomDocument xmlDocument, QDomElement domElement, QString filePath);
   void createOMNotebookCodeCell(LibraryTreeItem *pLibraryTreeItem, QDomDocument xmlDocument, QDomElement domElement);
   TransformationsWidget* showTransformationsWidget(QString fileName);
+  void findFileAndGoToLine(QString fileName, QString lineNumber);
   static void PlotCallbackFunction(void *p, int externalWindow, const char* filename, const char* title, const char* grid,
                                    const char* plotType, const char* logX, const char* logY, const char* xLabel, const char* yLabel,
                                    const char* x1, const char* x2, const char* y1, const char* y2, const char* curveWidth,
@@ -189,35 +193,38 @@ private:
   bool mDebug;
   OMCProxy *mpOMCProxy;
   bool mExitApplicationStatus;
-  OptionsDialog *mpOptionsDialog;
-  MessagesWidget *mpMessagesWidget;
   QDockWidget *mpMessagesDockWidget;
-  QFile mOutputFile;
   FileDataNotifier *mpOutputFileDataNotifier;
-  QFile mErrorFile;
   FileDataNotifier *mpErrorFileDataNotifier;
   LibraryWidget *mpLibraryWidget;
   QDockWidget *mpLibraryDockWidget;
+  GDBAdapter *mpGDBAdapter;
+  StackFramesWidget *mpStackFramesWidget;
+  QDockWidget *mpStackFramesDockWidget;
+  BreakpointsWidget *mpBreakpointsWidget;
+  QDockWidget *mpBreakpointsDockWidget;
+  LocalsWidget *mpLocalsWidget;
+  QDockWidget *mpLocalsDockWidget;
+  TargetOutputWidget *mpTargetOutputWidget;
+  QDockWidget *mpTargetOutputDockWidget;
+  GDBLoggerWidget *mpGDBLoggerWidget;
+  QDockWidget *mpGDBLoggerDockWidget;
   DocumentationWidget *mpDocumentationWidget;
   QDockWidget *mpDocumentationDockWidget;
+  PlotWindowContainer *mpPlotWindowContainer;
   VariablesWidget *mpVariablesWidget;
   QDockWidget *mpVariablesDockWidget;
   SimulationDialog *mpSimulationDialog;
   TLMCoSimulationDialog *mpTLMCoSimulationDialog;
-  PlotWindowContainer *mpPlotWindowContainer;
-  QList<Qt::WindowStates> mPlotWindowsStatesList;
-  QList<QByteArray> mPlotWindowsGeometriesList;
   ModelWidgetContainer *mpModelWidgetContainer;
-  DebuggerMainWindow *mpDebuggerMainWindow;
   WelcomePageWidget *mpWelcomePageWidget;
   AboutOMEditWidget *mpAboutOMEditDialog;
-  InfoBar *mpInfoBar;
   QStackedWidget *mpCentralStackedWidget;
-  QStatusBar *mpStatusBar;
   QProgressBar *mpProgressBar;
   Label *mpPointerXPositionLabel;
   Label *mpPointerYPositionLabel;
   QTabBar *mpPerspectiveTabbar;
+  QStatusBar *mpStatusBar;
   QTimer *mpAutoSaveTimer;
   // File Menu
   // Modelica File Actions
@@ -227,10 +234,11 @@ private:
   QAction *mpLoadModelicaLibraryAction;
   QAction *mpOpenResultFileAction;
   QAction *mpOpenTransformationFileAction;
-  // TLM File Actions
+  // MetaModel File Actions
   QAction *mpNewMetaModelFileAction;
   QAction *mpOpenMetaModelFileAction;
   QAction *mpLoadExternModelAction;
+  QAction *mpOpenDirectoryAction;
   QAction *mpSaveAction;
   QAction *mpSaveAsAction;
   QAction *mpSaveAllAction;
@@ -242,7 +250,7 @@ private:
   // Edit Menu
   QAction *mpUndoAction;
   QAction *mpRedoAction;
-  QAction *mpSearchClassesAction;
+  QAction *mpFilterClassesAction;
   QAction *mpCutAction;
   QAction *mpCopyAction;
   QAction *mpPasteAction;
@@ -251,7 +259,6 @@ private:
   QAction *mpResetZoomAction;
   QAction *mpZoomInAction;
   QAction *mpZoomOutAction;
-  QAction *mpShowAlgorithmicDebuggerAction;
   QAction *mpCloseWindowAction;
   QAction *mpCloseAllWindowsAction;
   QAction *mpCloseAllWindowsButThisAction;
@@ -265,15 +272,23 @@ private:
   QAction *mpSimulateModelAction;
   QAction *mpSimulateWithTransformationalDebuggerAction;
   QAction *mpSimulateWithAlgorithmicDebuggerAction;
+#if !defined(WITHOUT_OSG)
+  QAction *mpSimulateWithAnimationAction;
+#endif
   QAction *mpSimulationSetupAction;
   // FMI Menu
   QAction *mpExportFMUAction;
   QAction *mpImportFMUAction;
+  QAction *mpImportFMUModelDescriptionAction;
   // Export Menu
   QAction *mpExportXMLAction;
   QAction *mpExportFigaroAction;
+  // Debug Menu
+  QAction *mpDebugConfigurationsAction;
+  QAction *mpAttachDebuggerToRunningProcessAction;
   // Tools Menu
   QAction *mpShowOMCLoggerWidgetAction;
+  QAction *mpShowOpenModelicaCommandPromptAction;
   QAction *mpShowOMCDiffWidgetAction;
   QAction *mpExportToOMNotebookAction;
   QAction *mpImportFromOMNotebookAction;
@@ -307,12 +322,18 @@ private:
   QAction *mpReSimulateSetupAction;
   QAction *mpNewPlotWindowAction;
   QAction *mpNewParametricPlotWindowAction;
+#if !defined(WITHOUT_OSG)
+  QAction *mpNewAnimationWindowAction;
+#endif
   QAction *mpClearPlotWindowAction;
+  QAction *mpExportVariablesAction;
   // Other Actions
   QAction *mpExportAsImageAction;
   QAction *mpExportToClipboardAction;
   // TLM Simulation Action
+  QAction *mpSimulationParamsAction;
   QAction *mpFetchInterfaceDataAction;
+  QAction *mpAlignInterfacesAction;
   QAction *mpTLMCoSimulationAction;
   // Toolbars
   QMenu *mpRecentFilesMenu;
@@ -321,10 +342,12 @@ private:
   QToolBar *mpEditToolBar;
   QToolBar *mpViewToolBar;
   QToolBar *mpShapesToolBar;
-  QToolBar *mpSimulationToolBar;
   QToolBar *mpModelSwitcherToolBar;
   QToolButton *mpModelSwitcherToolButton;
   QMenu *mpModelSwitcherMenu;
+  QToolBar *mpCheckToolBar;
+  QToolBar *mpSimulationToolBar;
+  QToolBar *mpReSimulationToolBar;
   QToolBar *mpPlotToolBar;
   QToolBar *mpTLMSimulationToolbar;
   QHash<QString, TransformationsWidget*> mTransformationsWidgetHash;
@@ -338,6 +361,7 @@ public slots:
   void createNewMetaModelFile();
   void openMetaModelFile();
   void loadExternalModels();
+  void openDirectory();
   void loadSystemLibrary();
   void readOutputFile(qint64 bytes);
   void readErrorFile(qint64 bytes);
@@ -345,12 +369,11 @@ public slots:
   void clearRecentFilesList();
   void undo();
   void redo();
-  void focusSearchClasses();
+  void focusFilterClasses();
   void setShowGridLines(bool On);
   void resetZoom();
   void zoomIn();
   void zoomOut();
-  void showAlgorithmicDebugger();
   void closeWindow();
   void closeAllWindows();
   void closeAllWindowsButThis();
@@ -363,11 +386,14 @@ public slots:
   void simulateModel();
   void simulateModelWithTransformationalDebugger();
   void simulateModelWithAlgorithmicDebugger();
+  void simulateModelWithAnimation();
   void openSimulationDialog();
   void exportModelFMU();
   void importModelFMU();
+  void importFMUModelDescription();
   void exportModelXML();
   void exportModelFigaro();
+  void showOpenModelicaCommandPrompt();
   void exportModelToOMNotebook();
   void importModelfromOMNotebook();
   void importNgspiceNetlist();
@@ -389,30 +415,28 @@ public slots:
   void openAboutOMEdit();
   void toggleShapesButton();
   void openRecentModelWidget();
-  void reSimulateModel();
-  void showReSimulateSetup();
-  void addNewPlotWindow();
-  void addNewParametricPlotWindow();
-  void clearPlotWindow();
-  void showProgressBar();
-  void hideProgressBar();
   void updateModelSwitcherMenu(QMdiSubWindow *pSubWindow);
   void toggleAutoSave();
   void readInterfaceData(LibraryTreeItem *pLibraryTreeItem);
 private slots:
   void perspectiveTabChanged(int tabIndex);
+  void documentationDockWidgetVisibilityChanged(bool visible);
   void autoSave();
   void switchToWelcomePerspectiveSlot();
   void switchToModelingPerspectiveSlot();
   void switchToPlottingPerspectiveSlot();
+  void switchToAlgorithmicDebuggingPerspectiveSlot();
+  void showConfigureDialog();
+  void showAttachToProcessDialog();
 private:
   void createActions();
   void createToolbars();
   void createMenus();
-  void storePlotWindowsStateAndGeometry();
+  void autoSaveHelper(LibraryTreeItem *pLibraryTreeItem);
   void switchToWelcomePerspective();
   void switchToModelingPerspective();
   void switchToPlottingPerspective();
+  void switchToAlgorithmicDebuggingPerspective();
   void closeAllWindowsButThis(QMdiArea *pMdiArea);
   void tileSubWindows(QMdiArea *pMdiArea, bool horizontally);
   void fetchInterfaceDataHelper(LibraryTreeItem *pLibraryTreeItem);
@@ -423,16 +447,6 @@ protected:
   virtual void resizeEvent(QResizeEvent *event);
 };
 
-class InfoBar : public QFrame
-{
-public:
-  InfoBar(QWidget *pParent);
-  void showMessage(QString message);
-private:
-  Label *mpInfoLabel;
-  QToolButton *mpCloseButton;
-};
-
 class AboutOMEditWidget : public QWidget
 {
   Q_OBJECT
