@@ -32,7 +32,9 @@
  */
 
 #include "Commands.h"
-#include "ComponentProperties.h"
+#include "MainWindow.h"
+
+#include <QMessageBox>
 
 AddShapeCommand::AddShapeCommand(ShapeAnnotation *pShapeAnnotation, QUndoCommand *pParent)
   : QUndoCommand(pParent)
@@ -334,7 +336,7 @@ void UpdateComponentAttributesCommand::redo()
   QString isOuter = mNewComponentInfo.getOuter() ? "true" : "false";
   QString causality = mNewComponentInfo.getCausality();
 
-  OMCProxy *pOMCProxy = pModelWidget->getModelWidgetContainer()->getMainWindow()->getOMCProxy();
+  OMCProxy *pOMCProxy = MainWindow::instance()->getOMCProxy();
   // update component attributes
   if (pOMCProxy->setComponentProperties(modelName, mpComponent->getComponentInfo()->getName(), isFinal, flow, isProtected, isReplaceAble,
                                         variability, isInner, isOuter, causality)) {
@@ -367,7 +369,7 @@ void UpdateComponentAttributesCommand::redo()
       }
     }
   } else {
-    QMessageBox::critical(pModelWidget->getModelWidgetContainer()->getMainWindow(),
+    QMessageBox::critical(MainWindow::instance(),
                           QString(Helper::applicationName).append(" - ").append(Helper::error), pOMCProxy->getResult(), Helper::ok);
     pOMCProxy->printMessagesStringInternal();
   }
@@ -376,8 +378,24 @@ void UpdateComponentAttributesCommand::redo()
     QString comment = StringHandler::escapeString(mNewComponentInfo.getComment());
     if (pOMCProxy->setComponentComment(modelName, mpComponent->getComponentInfo()->getName(), comment)) {
       mpComponent->getComponentInfo()->setComment(comment);
+      mpComponent->componentCommentHasChanged();
+      if (mpComponent->getLibraryTreeItem()->isConnector()) {
+        if (mpComponent->getGraphicsView()->getViewType() == StringHandler::Icon) {
+          Component *pComponent = 0;
+          pComponent = mpComponent->getGraphicsView()->getModelWidget()->getDiagramGraphicsView()->getComponentObject(mpComponent->getName());
+          if (pComponent) {
+            pComponent->componentCommentHasChanged();
+          }
+        } else {
+          Component *pComponent = 0;
+          pComponent = mpComponent->getGraphicsView()->getModelWidget()->getIconGraphicsView()->getComponentObject(mpComponent->getName());
+          if (pComponent) {
+            pComponent->componentCommentHasChanged();
+          }
+        }
+      }
     } else {
-      QMessageBox::critical(pModelWidget->getModelWidgetContainer()->getMainWindow(),
+      QMessageBox::critical(MainWindow::instance(),
                             QString(Helper::applicationName).append(" - ").append(Helper::error), pOMCProxy->getResult(), Helper::ok);
       pOMCProxy->printMessagesStringInternal();
     }
@@ -386,6 +404,7 @@ void UpdateComponentAttributesCommand::redo()
   if (mpComponent->getComponentInfo()->getName().compare(mNewComponentInfo.getName()) != 0) {
     // if renameComponentInClass command is successful update the component with new name
     if (pOMCProxy->renameComponentInClass(modelName, mpComponent->getComponentInfo()->getName(), mNewComponentInfo.getName())) {
+      mpComponent->renameComponentInConnections(mNewComponentInfo.getName());
       mpComponent->getComponentInfo()->setName(mNewComponentInfo.getName());
       mpComponent->componentNameHasChanged();
       if (mpComponent->getLibraryTreeItem()->isConnector()) {
@@ -404,7 +423,7 @@ void UpdateComponentAttributesCommand::redo()
         }
       }
     } else {
-      QMessageBox::critical(pModelWidget->getModelWidgetContainer()->getMainWindow(),
+      QMessageBox::critical(MainWindow::instance(),
                             QString(Helper::applicationName).append(" - ").append(Helper::error), pOMCProxy->getResult(), Helper::ok);
       pOMCProxy->printMessagesStringInternal();
     }
@@ -414,7 +433,7 @@ void UpdateComponentAttributesCommand::redo()
     if (pOMCProxy->setComponentDimensions(modelName, mpComponent->getComponentInfo()->getName(), mNewComponentInfo.getArrayIndex())) {
       mpComponent->getComponentInfo()->setArrayIndex(mNewComponentInfo.getArrayIndex());
     } else {
-      QMessageBox::critical(pModelWidget->getModelWidgetContainer()->getMainWindow(),
+      QMessageBox::critical(MainWindow::instance(),
                             QString(Helper::applicationName).append(" - ").append(Helper::error), pOMCProxy->getResult(), Helper::ok);
       pOMCProxy->printMessagesStringInternal();
     }
@@ -460,7 +479,7 @@ void UpdateComponentAttributesCommand::undo()
   QString isOuter = mOldComponentInfo.getOuter() ? "true" : "false";
   QString causality = mOldComponentInfo.getCausality();
 
-  OMCProxy *pOMCProxy = pModelWidget->getModelWidgetContainer()->getMainWindow()->getOMCProxy();
+  OMCProxy *pOMCProxy = MainWindow::instance()->getOMCProxy();
   // update component attributes
   if (pOMCProxy->setComponentProperties(modelName, mpComponent->getComponentInfo()->getName(), isFinal, flow, isProtected, isReplaceAble,
                                         variability, isInner, isOuter, causality)) {
@@ -493,7 +512,7 @@ void UpdateComponentAttributesCommand::undo()
       }
     }
   } else {
-    QMessageBox::critical(pModelWidget->getModelWidgetContainer()->getMainWindow(),
+    QMessageBox::critical(MainWindow::instance(),
                           QString(Helper::applicationName).append(" - ").append(Helper::error), pOMCProxy->getResult(), Helper::ok);
     pOMCProxy->printMessagesStringInternal();
   }
@@ -502,8 +521,24 @@ void UpdateComponentAttributesCommand::undo()
     QString comment = StringHandler::escapeString(mOldComponentInfo.getComment());
     if (pOMCProxy->setComponentComment(modelName, mpComponent->getComponentInfo()->getName(), comment)) {
       mpComponent->getComponentInfo()->setComment(comment);
+      mpComponent->componentCommentHasChanged();
+      if (mpComponent->getLibraryTreeItem()->isConnector()) {
+        if (mpComponent->getGraphicsView()->getViewType() == StringHandler::Icon) {
+          Component *pComponent = 0;
+          pComponent = mpComponent->getGraphicsView()->getModelWidget()->getDiagramGraphicsView()->getComponentObject(mpComponent->getName());
+          if (pComponent) {
+            pComponent->componentCommentHasChanged();
+          }
+        } else {
+          Component *pComponent = 0;
+          pComponent = mpComponent->getGraphicsView()->getModelWidget()->getIconGraphicsView()->getComponentObject(mpComponent->getName());
+          if (pComponent) {
+            pComponent->componentCommentHasChanged();
+          }
+        }
+      }
     } else {
-      QMessageBox::critical(pModelWidget->getModelWidgetContainer()->getMainWindow(),
+      QMessageBox::critical(MainWindow::instance(),
                             QString(Helper::applicationName).append(" - ").append(Helper::error), pOMCProxy->getResult(), Helper::ok);
       pOMCProxy->printMessagesStringInternal();
     }
@@ -512,6 +547,7 @@ void UpdateComponentAttributesCommand::undo()
   if (mpComponent->getComponentInfo()->getName().compare(mOldComponentInfo.getName()) != 0) {
     // if renameComponentInClass command is successful update the component with new name
     if (pOMCProxy->renameComponentInClass(modelName, mpComponent->getComponentInfo()->getName(), mOldComponentInfo.getName())) {
+      mpComponent->renameComponentInConnections(mOldComponentInfo.getName());
       mpComponent->getComponentInfo()->setName(mOldComponentInfo.getName());
       mpComponent->componentNameHasChanged();
       if (mpComponent->getLibraryTreeItem()->isConnector()) {
@@ -530,7 +566,7 @@ void UpdateComponentAttributesCommand::undo()
         }
       }
     } else {
-      QMessageBox::critical(pModelWidget->getModelWidgetContainer()->getMainWindow(),
+      QMessageBox::critical(MainWindow::instance(),
                             QString(Helper::applicationName).append(" - ").append(Helper::error), pOMCProxy->getResult(), Helper::ok);
       pOMCProxy->printMessagesStringInternal();
     }
@@ -540,7 +576,7 @@ void UpdateComponentAttributesCommand::undo()
     if (pOMCProxy->setComponentDimensions(modelName, mpComponent->getComponentInfo()->getName(), mOldComponentInfo.getArrayIndex())) {
       mpComponent->getComponentInfo()->setArrayIndex(mOldComponentInfo.getArrayIndex());
     } else {
-      QMessageBox::critical(pModelWidget->getModelWidgetContainer()->getMainWindow(),
+      QMessageBox::critical(MainWindow::instance(),
                             QString(Helper::applicationName).append(" - ").append(Helper::error), pOMCProxy->getResult(), Helper::ok);
       pOMCProxy->printMessagesStringInternal();
     }
@@ -568,7 +604,7 @@ UpdateComponentParametersCommand::UpdateComponentParametersCommand(Component *pC
  */
 void UpdateComponentParametersCommand::redo()
 {
-  OMCProxy *pOMCProxy = mpComponent->getGraphicsView()->getModelWidget()->getModelWidgetContainer()->getMainWindow()->getOMCProxy();
+  OMCProxy *pOMCProxy = MainWindow::instance()->getOMCProxy();
   QString className = mpComponent->getGraphicsView()->getModelWidget()->getLibraryTreeItem()->getNameStructure();
   if (!mpComponent->getReferenceComponent()) {
     // remove all the modifiers of a component.
@@ -580,7 +616,7 @@ void UpdateComponentParametersCommand::redo()
       QString modifierKey = QString(mpComponent->getName()).append(".").append(componentModifier.key());
       pOMCProxy->setComponentModifierValue(className, modifierKey, modifierValue);
     }
-    mpComponent->getComponentInfo()->fetchModifiers(pOMCProxy, className);
+    mpComponent->getComponentInfo()->fetchModifiers(pOMCProxy, className, mpComponent);
   } else {
     QString inheritedClassName;
     inheritedClassName = mpComponent->getReferenceComponent()->getGraphicsView()->getModelWidget()->getLibraryTreeItem()->getNameStructure();
@@ -601,7 +637,7 @@ void UpdateComponentParametersCommand::redo()
  */
 void UpdateComponentParametersCommand::undo()
 {
-  OMCProxy *pOMCProxy = mpComponent->getGraphicsView()->getModelWidget()->getModelWidgetContainer()->getMainWindow()->getOMCProxy();
+  OMCProxy *pOMCProxy = MainWindow::instance()->getOMCProxy();
   QString className = mpComponent->getGraphicsView()->getModelWidget()->getLibraryTreeItem()->getNameStructure();
   if (!mpComponent->getReferenceComponent()) {
     // remove all the modifiers of a component.
@@ -613,7 +649,7 @@ void UpdateComponentParametersCommand::undo()
       QString modifierKey = QString(mpComponent->getName()).append(".").append(componentModifier.key());
       pOMCProxy->setComponentModifierValue(className, modifierKey, modifierValue);
     }
-    mpComponent->getComponentInfo()->fetchModifiers(pOMCProxy, className);
+    mpComponent->getComponentInfo()->fetchModifiers(pOMCProxy, className, mpComponent);
   } else {
     QString inheritedClassName;
     inheritedClassName = mpComponent->getReferenceComponent()->getGraphicsView()->getModelWidget()->getLibraryTreeItem()->getNameStructure();
@@ -731,10 +767,18 @@ void AddConnectionCommand::redo()
 {
   // Add the start component connection details.
   Component *pStartComponent = mpConnectionLineAnnotation->getStartComponent();
-  pStartComponent->getRootParentComponent()->addConnectionDetails(mpConnectionLineAnnotation);
+  if (pStartComponent->getRootParentComponent()) {
+    pStartComponent->getRootParentComponent()->addConnectionDetails(mpConnectionLineAnnotation);
+  } else {
+    pStartComponent->addConnectionDetails(mpConnectionLineAnnotation);
+  }
   // Add the end component connection details.
   Component *pEndComponent = mpConnectionLineAnnotation->getEndComponent();
-  pEndComponent->getRootParentComponent()->addConnectionDetails(mpConnectionLineAnnotation);
+  if (pEndComponent->getRootParentComponent()) {
+    pEndComponent->getRootParentComponent()->addConnectionDetails(mpConnectionLineAnnotation);
+  } else {
+    pEndComponent->addConnectionDetails(mpConnectionLineAnnotation);
+  }
   mpConnectionLineAnnotation->getGraphicsView()->addConnectionToList(mpConnectionLineAnnotation);
   mpConnectionLineAnnotation->getGraphicsView()->addItem(mpConnectionLineAnnotation);
   mpConnectionLineAnnotation->emitAdded();
@@ -751,10 +795,18 @@ void AddConnectionCommand::undo()
 {
   // Remove the start component connection details.
   Component *pStartComponent = mpConnectionLineAnnotation->getStartComponent();
-  pStartComponent->getRootParentComponent()->removeConnectionDetails(mpConnectionLineAnnotation);
+  if (pStartComponent->getRootParentComponent()) {
+    pStartComponent->getRootParentComponent()->removeConnectionDetails(mpConnectionLineAnnotation);
+  } else {
+    pStartComponent->removeConnectionDetails(mpConnectionLineAnnotation);
+  }
   // Remove the end component connection details.
   Component *pEndComponent = mpConnectionLineAnnotation->getEndComponent();
-  pEndComponent->getRootParentComponent()->removeConnectionDetails(mpConnectionLineAnnotation);
+  if (pEndComponent->getRootParentComponent()) {
+    pEndComponent->getRootParentComponent()->removeConnectionDetails(mpConnectionLineAnnotation);
+  } else {
+    pEndComponent->removeConnectionDetails(mpConnectionLineAnnotation);
+  }
   mpConnectionLineAnnotation->getGraphicsView()->deleteConnectionFromList(mpConnectionLineAnnotation);
   mpConnectionLineAnnotation->getGraphicsView()->removeItem(mpConnectionLineAnnotation);
   mpConnectionLineAnnotation->emitDeleted();
@@ -806,6 +858,43 @@ void UpdateConnectionCommand::undo()
   mpConnectionLineAnnotation->updateConnectionAnnotation();
 }
 
+UpdateMetaModelConnection::UpdateMetaModelConnection(LineAnnotation *pConnectionLineAnnotation, MetaModelConnection oldMetaModelConnection,
+                                                     MetaModelConnection newMetaModelConnection, QUndoCommand *pParent)
+  : QUndoCommand(pParent)
+{
+  mpConnectionLineAnnotation = pConnectionLineAnnotation;
+  mOldMetaModelConnection = oldMetaModelConnection;
+  mNewMetaModelConnection = newMetaModelConnection;
+  setText(QString("Update MetaModel Connection connect(%1, %2)").arg(mpConnectionLineAnnotation->getStartComponentName(),
+                                                                     mpConnectionLineAnnotation->getEndComponentName()));
+}
+
+/*!
+ * \brief UpdateMetaModelConnection::redo
+ * Redo the UpdateMetaModelConnection.
+ */
+void UpdateMetaModelConnection::redo()
+{
+  mpConnectionLineAnnotation->setDelay(mNewMetaModelConnection.mDelay);
+  mpConnectionLineAnnotation->setZf(mNewMetaModelConnection.mZf);
+  mpConnectionLineAnnotation->setZfr(mNewMetaModelConnection.mZfr);
+  mpConnectionLineAnnotation->setAlpha(mNewMetaModelConnection.mAlpha);
+  mpConnectionLineAnnotation->getGraphicsView()->updateConnectionInClass(mpConnectionLineAnnotation);
+}
+
+/*!
+ * \brief UpdateMetaModelConnection::undo
+ * Undo the UpdateMetaModelConnection.
+ */
+void UpdateMetaModelConnection::undo()
+{
+  mpConnectionLineAnnotation->setDelay(mOldMetaModelConnection.mDelay);
+  mpConnectionLineAnnotation->setZf(mOldMetaModelConnection.mZf);
+  mpConnectionLineAnnotation->setZfr(mOldMetaModelConnection.mZfr);
+  mpConnectionLineAnnotation->setAlpha(mOldMetaModelConnection.mAlpha);
+  mpConnectionLineAnnotation->getGraphicsView()->updateConnectionInClass(mpConnectionLineAnnotation);
+}
+
 DeleteConnectionCommand::DeleteConnectionCommand(LineAnnotation *pConnectionLineAnnotation, QUndoCommand *pParent)
   : QUndoCommand(pParent)
 {
@@ -827,8 +916,8 @@ void DeleteConnectionCommand::redo()
   }
   // Remove the end component connection details.
   Component *pEndComponent = mpConnectionLineAnnotation->getEndComponent();
-  if (pEndComponent->getParentComponent()) {
-    pEndComponent->getParentComponent()->removeConnectionDetails(mpConnectionLineAnnotation);
+  if (pEndComponent->getRootParentComponent()) {
+    pEndComponent->getRootParentComponent()->removeConnectionDetails(mpConnectionLineAnnotation);
   } else {
     pEndComponent->removeConnectionDetails(mpConnectionLineAnnotation);
   }
@@ -853,8 +942,8 @@ void DeleteConnectionCommand::undo()
   }
   // Add the end component connection details.
   Component *pEndComponent = mpConnectionLineAnnotation->getEndComponent();
-  if (pEndComponent->getParentComponent()) {
-    pEndComponent->getParentComponent()->addConnectionDetails(mpConnectionLineAnnotation);
+  if (pEndComponent->getRootParentComponent()) {
+    pEndComponent->getRootParentComponent()->addConnectionDetails(mpConnectionLineAnnotation);
   } else {
     pEndComponent->addConnectionDetails(mpConnectionLineAnnotation);
   }
@@ -864,44 +953,23 @@ void DeleteConnectionCommand::undo()
   mpConnectionLineAnnotation->getGraphicsView()->addConnectionToClass(mpConnectionLineAnnotation);
 }
 
-UpdateClassExperimentAnnotationCommand::UpdateClassExperimentAnnotationCommand(MainWindow *pMainWindow, LibraryTreeItem *pLibraryTreeItem,
-                                                                               QString oldExperimentAnnotation, QString newExperimentAnnotaiton,
-                                                                               QUndoCommand *pParent)
-  : QUndoCommand(pParent)
-{
-  mpMainWindow = pMainWindow;
-  mpLibraryTreeItem = pLibraryTreeItem;
-  mOldExperimentAnnotation = oldExperimentAnnotation;
-  mNewExperimentAnnotation = newExperimentAnnotaiton;
-  setText(QString("Update %1 experiment annotation").arg(mpLibraryTreeItem->getNameStructure()));
-}
-
-/*!
- * \brief UpdateClassExperimentAnnotationCommand::redo
- * Redo the UpdateClassExperimentAnnotationCommand.
- */
-void UpdateClassExperimentAnnotationCommand::redo()
-{
-  mpMainWindow->getOMCProxy()->addClassAnnotation(mpLibraryTreeItem->getNameStructure(), mNewExperimentAnnotation);
-}
-
-/*!
- * \brief UpdateClassExperimentAnnotationCommand::undo
- * Undo the UpdateClassExperimentAnnotationCommand.
- */
-void UpdateClassExperimentAnnotationCommand::undo()
-{
-  mpMainWindow->getOMCProxy()->addClassAnnotation(mpLibraryTreeItem->getNameStructure(), mOldExperimentAnnotation);
-}
-
 UpdateCoOrdinateSystemCommand::UpdateCoOrdinateSystemCommand(GraphicsView *pGraphicsView, CoOrdinateSystem oldCoOrdinateSystem,
-                                                             CoOrdinateSystem newCoOrdinateSystem, bool copyProperties, QUndoCommand *pParent)
+                                                             CoOrdinateSystem newCoOrdinateSystem, bool copyProperties, QString oldVersion,
+                                                             QString newVersion, QString oldUsesAnnotationString,
+                                                             QString newUsesAnnotationString, QString oldOMCFlags, QString newOMCFlags,
+                                                             QUndoCommand *pParent)
   : QUndoCommand(pParent)
 {
   mpGraphicsView = pGraphicsView;
   mOldCoOrdinateSystem = oldCoOrdinateSystem;
   mNewCoOrdinateSystem = newCoOrdinateSystem;
   mCopyProperties = copyProperties;
+  mOldVersion = oldVersion;
+  mNewVersion = newVersion;
+  mOldUsesAnnotationString = oldUsesAnnotationString;
+  mNewUsesAnnotationString = newUsesAnnotationString;
+  mOldOMCFlags = oldOMCFlags;
+  mNewOMCFlags = newOMCFlags;
   setText(QString("Update %1 CoOrdinate System").arg(mpGraphicsView->getModelWidget()->getLibraryTreeItem()->getNameStructure()));
 }
 
@@ -918,7 +986,8 @@ void UpdateCoOrdinateSystemCommand::redo()
   qreal top = mNewCoOrdinateSystem.getExtent().at(1).y();
   mpGraphicsView->setExtentRectangle(left, bottom, right, top);
   mpGraphicsView->addClassAnnotation();
-  mpGraphicsView->scene()->update();
+  mpGraphicsView->fitInViewInternal();
+  mpGraphicsView->getModelWidget()->getLibraryTreeItem()->emitCoOrdinateSystemUpdated(mpGraphicsView);
   // if copy properties is true
   if (mCopyProperties) {
     GraphicsView *pGraphicsView;
@@ -930,8 +999,23 @@ void UpdateCoOrdinateSystemCommand::redo()
     pGraphicsView->mCoOrdinateSystem = mNewCoOrdinateSystem;
     pGraphicsView->setExtentRectangle(left, bottom, right, top);
     pGraphicsView->addClassAnnotation();
-    pGraphicsView->scene()->update();
+    pGraphicsView->fitInViewInternal();
+    pGraphicsView->getModelWidget()->getLibraryTreeItem()->emitCoOrdinateSystemUpdated(pGraphicsView);
   }
+  OMCProxy *pOMCProxy = MainWindow::instance()->getOMCProxy();
+  // only add version and uses annotation to top level class.
+  if (mpGraphicsView->getModelWidget()->getLibraryTreeItem()->isTopLevel()) {
+    // version
+    QString versionAnnotation = QString("annotate=version(\"%1\")").arg(mNewVersion);
+    if (pOMCProxy->addClassAnnotation(mpGraphicsView->getModelWidget()->getLibraryTreeItem()->getNameStructure(), versionAnnotation)) {
+      mpGraphicsView->getModelWidget()->getLibraryTreeItem()->mClassInformation.version = mNewVersion;
+    }
+    // uses annotation
+    pOMCProxy->addClassAnnotation(mpGraphicsView->getModelWidget()->getLibraryTreeItem()->getNameStructure(), mNewUsesAnnotationString);
+  }
+  // omc flags
+  QString flagsAnnotation = QString("annotate=__OpenModelica_commandLineOptions(\"%1\")").arg(mNewOMCFlags);
+  pOMCProxy->addClassAnnotation(mpGraphicsView->getModelWidget()->getLibraryTreeItem()->getNameStructure(), flagsAnnotation);
 }
 
 /*!
@@ -945,9 +1029,15 @@ void UpdateCoOrdinateSystemCommand::undo()
   qreal bottom = mOldCoOrdinateSystem.getExtent().at(0).y();
   qreal right = mOldCoOrdinateSystem.getExtent().at(1).x();
   qreal top = mOldCoOrdinateSystem.getExtent().at(1).y();
-  mpGraphicsView->setExtentRectangle(left, bottom, right, top);
+
+  if (!mpGraphicsView->mCoOrdinateSystem.isValid()) {
+    mpGraphicsView->getModelWidget()->drawBaseCoOrdinateSystem(mpGraphicsView->getModelWidget(), mpGraphicsView);
+  } else {
+    mpGraphicsView->setExtentRectangle(left, bottom, right, top);
+  }
   mpGraphicsView->addClassAnnotation();
-  mpGraphicsView->scene()->update();
+  mpGraphicsView->fitInViewInternal();
+  mpGraphicsView->getModelWidget()->getLibraryTreeItem()->emitCoOrdinateSystemUpdated(mpGraphicsView);
   // if copy properties is true
   if (mCopyProperties) {
     GraphicsView *pGraphicsView;
@@ -957,8 +1047,246 @@ void UpdateCoOrdinateSystemCommand::undo()
       pGraphicsView = mpGraphicsView->getModelWidget()->getIconGraphicsView();
     }
     pGraphicsView->mCoOrdinateSystem = mOldCoOrdinateSystem;
-    pGraphicsView->setExtentRectangle(left, bottom, right, top);
+    if (!pGraphicsView->mCoOrdinateSystem.isValid()) {
+      pGraphicsView->getModelWidget()->drawBaseCoOrdinateSystem(pGraphicsView->getModelWidget(), pGraphicsView);
+    } else {
+      pGraphicsView->setExtentRectangle(left, bottom, right, top);
+    }
     pGraphicsView->addClassAnnotation();
-    pGraphicsView->scene()->update();
+    pGraphicsView->fitInViewInternal();
+    pGraphicsView->getModelWidget()->getLibraryTreeItem()->emitCoOrdinateSystemUpdated(pGraphicsView);
+  }
+  OMCProxy *pOMCProxy = MainWindow::instance()->getOMCProxy();
+  // only add version and uses annotation to top level class.
+  if (mpGraphicsView->getModelWidget()->getLibraryTreeItem()->isTopLevel()) {
+    // version
+    QString versionAnnotation = QString("annotate=version(\"%1\")").arg(mOldVersion);
+    if (pOMCProxy->addClassAnnotation(mpGraphicsView->getModelWidget()->getLibraryTreeItem()->getNameStructure(), versionAnnotation)) {
+      mpGraphicsView->getModelWidget()->getLibraryTreeItem()->mClassInformation.version = mOldVersion;
+    }
+    // uses annotation
+    pOMCProxy->addClassAnnotation(mpGraphicsView->getModelWidget()->getLibraryTreeItem()->getNameStructure(), mOldUsesAnnotationString);
+  }
+  // omc flags
+  QString flagsAnnotation = QString("annotate=__OpenModelica_commandLineOptions(\"%1\")").arg(mOldOMCFlags);
+  pOMCProxy->addClassAnnotation(mpGraphicsView->getModelWidget()->getLibraryTreeItem()->getNameStructure(), flagsAnnotation);
+}
+
+/*!
+ * \class UpdateClassAnnotationCommand
+ * \brief A class for updating the class annotation e.g. experiment, documentation etc.
+ */
+/*!
+ * \brief UpdateClassAnnotationCommand::UpdateClassAnnotationCommand
+ * \param pLibraryTreeItem
+ * \param oldAnnotation
+ * \param newAnnotaiton
+ * \param pParent
+ */
+UpdateClassAnnotationCommand::UpdateClassAnnotationCommand(LibraryTreeItem *pLibraryTreeItem, QString oldAnnotation,
+                                                           QString newAnnotaiton, QUndoCommand *pParent)
+  : QUndoCommand(pParent)
+{
+  mpLibraryTreeItem = pLibraryTreeItem;
+  mOldAnnotation = oldAnnotation;
+  mNewAnnotation = newAnnotaiton;
+  setText(QString("Update %1 annotation").arg(mpLibraryTreeItem->getNameStructure()));
+}
+
+/*!
+ * \brief UpdateClassAnnotationCommand::redo
+ * Redo the UpdateClassAnnotationCommand.
+ */
+void UpdateClassAnnotationCommand::redo()
+{
+  MainWindow::instance()->getOMCProxy()->addClassAnnotation(mpLibraryTreeItem->getNameStructure(), mNewAnnotation);
+}
+
+/*!
+ * \brief UpdateClassAnnotationCommand::undo
+ * Undo the UpdateClassAnnotationCommand.
+ */
+void UpdateClassAnnotationCommand::undo()
+{
+  MainWindow::instance()->getOMCProxy()->addClassAnnotation(mpLibraryTreeItem->getNameStructure(), mOldAnnotation);
+}
+
+UpdateClassSimulationFlagsAnnotationCommand::UpdateClassSimulationFlagsAnnotationCommand(LibraryTreeItem *pLibraryTreeItem,
+                                                                                         QString oldSimulationFlags,
+                                                                                         QString newSimulationFlags, QUndoCommand *pParent)
+  : QUndoCommand(pParent)
+{
+  mpLibraryTreeItem = pLibraryTreeItem;
+  mOldSimulationFlags = oldSimulationFlags;
+  mNewSimulationFlags = newSimulationFlags;
+  setText(QString("Update %1 simulation flags annotation").arg(mpLibraryTreeItem->getNameStructure()));
+}
+
+/*!
+ * \brief UpdateClassSimulationFlagsAnnotationCommand::redo
+ * Redo the UpdateClassSimulationFlagsAnnotationCommand.
+ */
+void UpdateClassSimulationFlagsAnnotationCommand::redo()
+{
+  MainWindow::instance()->getOMCProxy()->addClassAnnotation(mpLibraryTreeItem->getNameStructure(), mNewSimulationFlags);
+}
+
+/*!
+ * \brief UpdateClassSimulationFlagsAnnotationCommand::undo
+ * Undo the UpdateClassSimulationFlagsAnnotationCommand.
+ */
+void UpdateClassSimulationFlagsAnnotationCommand::undo()
+{
+  MainWindow::instance()->getOMCProxy()->addClassAnnotation(mpLibraryTreeItem->getNameStructure(), mOldSimulationFlags);
+}
+
+UpdateSubModelAttributesCommand::UpdateSubModelAttributesCommand(Component *pComponent, const ComponentInfo &oldComponentInfo,
+                                                                 const ComponentInfo &newComponentInfo, QUndoCommand *pParent)
+  : QUndoCommand(pParent)
+{
+  mpComponent = pComponent;
+  mOldComponentInfo.updateComponentInfo(&oldComponentInfo);
+  mNewComponentInfo.updateComponentInfo(&newComponentInfo);
+  setText(QString("Update SubModel %1 Attributes").arg(mpComponent->getName()));
+}
+
+/*!
+ * \brief UpdateSubModelAttributesCommand::redo
+ * Redo the UpdateSubModelAttributesCommand.
+ */
+void UpdateSubModelAttributesCommand::redo()
+{
+  MetaModelEditor *pMetaModelEditor = dynamic_cast<MetaModelEditor*>(mpComponent->getGraphicsView()->getModelWidget()->getEditor());
+  pMetaModelEditor->updateSubModelParameters(mpComponent->getName(), mNewComponentInfo.getStartCommand(),
+                                             mNewComponentInfo.getExactStep() ? "true" : "false", mNewComponentInfo.getGeometryFile());
+  mpComponent->getComponentInfo()->setStartCommand(mNewComponentInfo.getStartCommand());
+  mpComponent->getComponentInfo()->setExactStep(mNewComponentInfo.getExactStep());
+  mpComponent->getComponentInfo()->setGeometryFile(mNewComponentInfo.getGeometryFile());
+}
+
+/*!
+ * \brief UpdateSubModelAttributesCommand::undo
+ * Undo the UpdateSubModelAttributesCommand.
+ */
+void UpdateSubModelAttributesCommand::undo()
+{
+  MetaModelEditor *pMetaModelEditor = dynamic_cast<MetaModelEditor*>(mpComponent->getGraphicsView()->getModelWidget()->getEditor());
+  pMetaModelEditor->updateSubModelParameters(mpComponent->getName(), mOldComponentInfo.getStartCommand(),
+                                             mOldComponentInfo.getExactStep() ? "true" : "false", mOldComponentInfo.getGeometryFile());
+  mpComponent->getComponentInfo()->setStartCommand(mOldComponentInfo.getStartCommand());
+  mpComponent->getComponentInfo()->setExactStep(mOldComponentInfo.getExactStep());
+  mpComponent->getComponentInfo()->setGeometryFile(mOldComponentInfo.getGeometryFile());
+}
+
+UpdateSimulationParamsCommand::UpdateSimulationParamsCommand(LibraryTreeItem *pLibraryTreeItem, QString oldStartTime, QString newStartTime, QString oldStopTime,
+                                                             QString newStopTime, QUndoCommand *pParent )
+  : QUndoCommand(pParent)
+{
+  mpLibraryTreeItem = pLibraryTreeItem;
+  mOldStartTime = oldStartTime;
+  mNewStartTime = newStartTime;
+  mOldStopTime = oldStopTime;
+  mNewStopTime = newStopTime;
+  setText(QString("Update %1 simulation parameter").arg(mpLibraryTreeItem->getNameStructure()));
+}
+
+/*!
+ * \brief UpdateSimulationParamsCommand::redo
+ * Redo the UpdateSimulationParamsCommand.
+ */
+void UpdateSimulationParamsCommand::redo()
+{
+  MetaModelEditor *pMetaModelEditor = dynamic_cast<MetaModelEditor*>(mpLibraryTreeItem->getModelWidget()->getEditor());
+  pMetaModelEditor->updateSimulationParams(mNewStartTime, mNewStopTime);
+}
+
+/*!
+ * \brief UpdateSimulationParamsCommand::undo
+ * Undo the UpdateSimulationParamsCommand.
+ */
+void UpdateSimulationParamsCommand::undo()
+{
+  MetaModelEditor *pMetaModelEditor = dynamic_cast<MetaModelEditor*>(mpLibraryTreeItem->getModelWidget()->getEditor());
+  pMetaModelEditor->updateSimulationParams(mOldStartTime, mOldStopTime);
+}
+
+/*!
+ * \brief AlignInterfacesCommand::AlignInterfacesCommand
+ * \param pEditor
+ * \param oldText
+ * \param newText
+ * \param pParent
+ */
+AlignInterfacesCommand::AlignInterfacesCommand(MetaModelEditor *pMetaModelEditor, QString fromInterface, QString toInterface,
+                                               QGenericMatrix<3,1,double> oldPos, QGenericMatrix<3,1,double> oldRot,
+                                               QGenericMatrix<3,1,double> newPos, QGenericMatrix<3,1,double> newRot,
+                                               LineAnnotation *pConnectionLineAnnotation, QUndoCommand *pParent)
+  : QUndoCommand(pParent)
+{
+  mpMetaModelEditor = pMetaModelEditor;
+  mFromInterface = fromInterface;
+  mToInterface = toInterface;
+  mOldPos = oldPos;
+  mOldRot = oldRot;
+  mNewPos = newPos;
+  mNewRot = newRot;
+  mpConnectionLineAnnotation = pConnectionLineAnnotation;
+}
+
+/*!
+ * \brief AlignInterfacesCommand::redo
+ * Redo the align interfaces command
+ */
+void AlignInterfacesCommand::redo()
+{
+  mpMetaModelEditor->updateSubModelOrientation(mFromInterface.split(".").first(), mNewPos, mNewRot);
+  //qDebug() << mpMetaModelEditor->interfacesAligned(mFromInterface, mToInterface);
+  if (mpConnectionLineAnnotation) {
+    mpConnectionLineAnnotation->setAligned(mpMetaModelEditor->interfacesAligned(mFromInterface, mToInterface));
   }
 }
+
+/*!
+ * \brief AlignInterfacesCommand::undo
+ * Undo the align interfaces command
+ */
+void AlignInterfacesCommand::undo()
+{
+  mpMetaModelEditor->updateSubModelOrientation(mFromInterface.split(".").first(), mOldPos, mOldRot);
+  //qDebug() << mpMetaModelEditor->interfacesAligned(mFromInterface, mToInterface);
+  if (mpConnectionLineAnnotation) {
+    mpConnectionLineAnnotation->setAligned(mpMetaModelEditor->interfacesAligned(mFromInterface, mToInterface));
+  }
+}
+
+RenameMetaModelCommand::RenameMetaModelCommand(MetaModelEditor *pMetaModelEditor, QString oldMetaModelName, QString newMetaModelName,
+                                               QUndoCommand *pParent)
+  : QUndoCommand(pParent)
+{
+  mpMetaModelEditor = pMetaModelEditor;
+  mOldMetaModelName = oldMetaModelName;
+  mNewMetaModelName = newMetaModelName;
+  setText(QString("Rename metamodel %1").arg(mpMetaModelEditor->getModelWidget()->getLibraryTreeItem()->getName()));
+}
+
+/*!
+ * \brief RenameMetaModelCommand::redo
+ * Redo the rename metamodel command
+ */
+void RenameMetaModelCommand::redo()
+{
+  mpMetaModelEditor->setMetaModelName(mNewMetaModelName);
+  mpMetaModelEditor->getModelWidget()->getLibraryTreeItem()->setName(mNewMetaModelName);
+  mpMetaModelEditor->getModelWidget()->setWindowTitle(mNewMetaModelName);
+}
+
+/*!
+ * \brief RenameMetaModelCommand::undo
+ * Undo the rename metamodel command
+ */
+void RenameMetaModelCommand::undo()
+{
+  mpMetaModelEditor->setMetaModelName(mOldMetaModelName);
+  mpMetaModelEditor->getModelWidget()->getLibraryTreeItem()->setName(mOldMetaModelName);
+  mpMetaModelEditor->getModelWidget()->setWindowTitle(mOldMetaModelName);
+}
