@@ -29,33 +29,34 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- * RCS: $Id$
- *
  */
 
 #ifndef MODELWIDGETCONTAINER_H
 #define MODELWIDGETCONTAINER_H
 
+#include "CoOrdinateSystem.h"
+#include "Component/Component.h"
+#include "Util/StringHandler.h"
+#include "Util/Helper.h"
+#include "Editors/BaseEditor.h"
+#include "Editors/ModelicaEditor.h"
+#include "Editors/MetaModelEditor.h"
+#include "Editors/CEditor.h"
+#include "Editors/TextEditor.h"
+#include "Editors/MetaModelicaEditor.h"
+#include "LibraryTreeWidget.h"
+
 #include <QGraphicsView>
 #include <QGraphicsScene>
 #include <QStatusBar>
 #include <QListWidget>
 #include <QMdiArea>
-#include <map>
 #include <QtWebKit>
 #include <QtXmlPatterns>
-
-#include "CoOrdinateSystem.h"
-#include "Component.h"
-#include "StringHandler.h"
-#include "Helper.h"
-#include "BaseEditor.h"
-#include "ModelicaTextEditor.h"
-#include "TLMEditor.h"
-#include "TextEditor.h"
+#include <QSplitter>
+#include <QUndoStack>
+#include <QUndoView>
 
 class ModelWidget;
 class ComponentInfo;
@@ -109,6 +110,8 @@ private:
   TextAnnotation *mpTextShapeAnnotation;
   BitmapAnnotation *mpBitmapShapeAnnotation;
   QAction *mpPropertiesAction;
+  QAction *mpRenameAction;
+  QAction *mpSimulationParamsAction;
   QAction *mpManhattanizeAction;
   QAction *mpDeleteAction;
   QAction *mpBringToFrontAction;
@@ -120,6 +123,7 @@ private:
   QAction *mpRotateAntiClockwiseAction;
   QAction *mpFlipHorizontalAction;
   QAction *mpFlipVerticalAction;
+  QAction *mpAttributesAction;
 public:
   GraphicsView(StringHandler::ViewType viewType, ModelWidget *parent);
   CoOrdinateSystem mCoOrdinateSystem;
@@ -183,7 +187,8 @@ public:
   QList<LineAnnotation*> getConnectionsList() {return mConnectionsList;}
   QList<LineAnnotation*> getInheritedConnectionsList() {return mInheritedConnectionsList;}
   void addConnectionToClass(LineAnnotation *pConnectionLineAnnotation);
-  void deleteConnectionFromClass(LineAnnotation *pConnectonLineAnnotation);
+  void deleteConnectionFromClass(LineAnnotation *pConnectionLineAnnotation);
+  void updateConnectionInClass(LineAnnotation *pConnectionLineAnnotation);
   void addConnectionToList(LineAnnotation *pConnectionLineAnnotation) {mConnectionsList.append(pConnectionLineAnnotation);}
   void addInheritedConnectionToList(LineAnnotation *pConnectionLineAnnotation) {mInheritedConnectionsList.append(pConnectionLineAnnotation);}
   void deleteConnectionFromList(LineAnnotation *pConnectionLineAnnotation) {mConnectionsList.removeOne(pConnectionLineAnnotation);}
@@ -214,6 +219,7 @@ public:
   bool hasAnnotation();
   void addItem(QGraphicsItem *pGraphicsItem);
   void removeItem(QGraphicsItem *pGraphicsItem);
+  void fitInViewInternal();
 private:
   void createActions();
   bool isClassDroppedOnItself(LibraryTreeItem *pLibraryTreeItem);
@@ -229,6 +235,8 @@ signals:
   void keyPressDelete();
   void keyPressRotateClockwise();
   void keyPressRotateAntiClockwise();
+  void keyPressFlipHorizontal();
+  void keyPressFlipVertical();
   void keyPressUp();
   void keyPressShiftUp();
   void keyPressCtrlUp();
@@ -251,8 +259,10 @@ public slots:
   void zoomOut();
   void selectAll();
   void clearSelection();
-  void addClassAnnotation();
+  void addClassAnnotation(bool alwaysAdd = true);
   void showGraphicsViewProperties();
+  void showRenameDialog();
+  void showSimulationParamsDialog();
   void manhattanizeItems();
   void deleteItems();
   void duplicateItems();
@@ -280,12 +290,11 @@ class WelcomePageWidget : public QWidget
 {
   Q_OBJECT
 public:
-  WelcomePageWidget(MainWindow *parent = 0);
+  WelcomePageWidget(QWidget *pParent = 0);
   void addRecentFilesListItems();
   QFrame* getLatestNewsFrame();
   QSplitter* getSplitter();
 private:
-  MainWindow *mpMainWindow;
   QFrame *mpMainFrame;
   QFrame *mpTopFrame;
   Label *mpPixmapLabel;
@@ -315,14 +324,14 @@ private slots:
 };
 
 class ModelWidgetContainer;
-class ModelicaTextHighlighter;
-class TLMHighlighter;
+class ModelicaHighlighter;
+class MetaModelHighlighter;
 class Label;
 class ModelWidget : public QWidget
 {
   Q_OBJECT
 public:
-  ModelWidget(LibraryTreeItem* pLibraryTreeItem, ModelWidgetContainer *pModelWidgetContainer, QString text);
+  ModelWidget(LibraryTreeItem* pLibraryTreeItem, ModelWidgetContainer *pModelWidgetContainer);
   ModelWidgetContainer* getModelWidgetContainer() {return mpModelWidgetContainer;}
   void setLibraryTreeItem(LibraryTreeItem *pLibraryTreeItem) {mpLibraryTreeItem = pLibraryTreeItem;}
   LibraryTreeItem* getLibraryTreeItem() {return mpLibraryTreeItem;}
@@ -334,6 +343,7 @@ public:
   GraphicsView* getIconGraphicsView() {return mpIconGraphicsView;}
   QUndoStack* getUndoStack() {return mpUndoStack;}
   BaseEditor* getEditor() {return mpEditor;}
+  void setModelClassPathLabel(QString path) {mpModelClassPathLabel->setText(path);}
   void setModelFilePathLabel(QString path) {mpModelFilePathLabel->setText(path);}
   Label* getCursorPositionLabel() {return mpCursorPositionLabel;}
   bool isLoadedWidgetComponents() {return mCreateModelWidgetComponents;}
@@ -341,26 +351,31 @@ public:
   void removeInheritedClass(LibraryTreeItem *pLibraryTreeItem) {mInheritedClassesList.removeOne(pLibraryTreeItem);}
   void clearInheritedClasses() {mInheritedClassesList.clear();}
   QList<LibraryTreeItem*> getInheritedClassesList() {return mInheritedClassesList;}
-  QMap<QString, QMap<QString, QString> > getExtendsModifiersMap() {return mExtendsModifiersMap;}
   QMap<QString, QString> getExtendsModifiersMap(QString extendsClass);
   void fetchExtendsModifiers(QString extendsClass);
   void reDrawModelWidgetInheritedClasses();
+  void drawBaseCoOrdinateSystem(ModelWidget *pModelWidget, GraphicsView *pGraphicsView);
   ShapeAnnotation* createNonExistingInheritedShape(GraphicsView *pGraphicsView);
   ShapeAnnotation* createInheritedShape(ShapeAnnotation *pShapeAnnotation, GraphicsView *pGraphicsView);
   Component* createInheritedComponent(Component *pComponent, GraphicsView *pGraphicsView);
   LineAnnotation* createInheritedConnection(LineAnnotation *pConnectionLineAnnotation);
+  void loadComponents();
   void loadDiagramView();
   void loadConnections();
   void createModelWidgetComponents();
   Component* getConnectorComponent(Component *pConnectorComponent, QString connectorName);
   void reDrawModelWidget();
-  bool validateText();
-  bool modelicaEditorTextChanged();
+  bool validateText(LibraryTreeItem **pLibraryTreeItem);
+  bool modelicaEditorTextChanged(LibraryTreeItem **pLibraryTreeItem);
   void updateChildClasses(LibraryTreeItem *pLibraryTreeItem);
   void clearSelection();
   void updateClassAnnotationIfNeeded();
-  void updateModelicaText();
+  void updateModelText();
+  void updateModelicaTextManually(QString contents);
   void updateUndoRedoActions();
+  void updateDynamicResults(QString resultFileName);
+  QString getResultFileName() {return mResultFileName;}
+  void writeVisualXMLFile();
 private:
   ModelWidgetContainer *mpModelWidgetContainer;
   LibraryTreeItem *mpLibraryTreeItem;
@@ -372,6 +387,7 @@ private:
   Label *mpReadOnlyLabel;
   Label *mpModelicaTypeLabel;
   Label *mpViewTypeLabel;
+  Label *mpModelClassPathLabel;
   Label *mpModelFilePathLabel;
   Label *mpCursorPositionLabel;
   QToolButton *mpFileLockToolButton;
@@ -382,9 +398,8 @@ private:
   QUndoStack *mpUndoStack;
   QUndoView *mpUndoView;
   BaseEditor *mpEditor;
-  ModelicaTextHighlighter *mpModelicaTextHighlighter;
-  TLMHighlighter *mpTLMHighlighter;
   QStatusBar *mpModelStatusBar;
+  bool mComponentsLoaded;
   bool mDiagramViewLoaded;
   bool mConnectionsLoaded;
   bool mCreateModelWidgetComponents;
@@ -393,6 +408,7 @@ private:
   QList<LibraryTreeItem*> mInheritedClassesList;
   QList<ComponentInfo*> mComponentsList;
   QStringList mComponentsAnnotationsList;
+  QString mResultFileName;
 
   void getModelInheritedClasses();
   void drawModelInheritedClassShapes(ModelWidget *pModelWidget, StringHandler::ViewType viewType);
@@ -406,8 +422,10 @@ private:
   void drawModelInheritedClassConnections(ModelWidget *pModelWidget);
   void removeInheritedClassConnections();
   void getModelConnections();
-  void getTLMComponents();
-  void getTLMConnections();
+  void detectMultipleDeclarations();
+  QString getMetaModelName();
+  void getMetaModelSubModels();
+  void getMetaModelConnections();
 private slots:
   void showIconView(bool checked);
   void showDiagramView(bool checked);
@@ -415,27 +433,28 @@ private slots:
 public slots:
   void makeFileWritAble();
   void showDocumentationView();
-  bool TLMEditorTextChanged();
+  bool metaModelEditorTextChanged();
   void handleCanUndoChanged(bool canUndo);
   void handleCanRedoChanged(bool canRedo);
+  void removeDynamicResults(QString resultFileName = "");
 protected:
   virtual void closeEvent(QCloseEvent *event);
 };
 
-class MdiArea;
-class ModelWidgetContainer : public MdiArea
+class ModelWidgetContainer : public QMdiArea
 {
   Q_OBJECT
 public:
-  ModelWidgetContainer(MainWindow *pParent);
+  ModelWidgetContainer(QWidget *pParent = 0);
   void addModelWidget(ModelWidget *pModelWidget, bool checkPreferedView = true);
   ModelWidget* getCurrentModelWidget();
+  ModelWidget* getModelWidget(const QString &className);
   QMdiSubWindow* getCurrentMdiSubWindow();
   QMdiSubWindow* getMdiSubWindow(ModelWidget *pModelWidget);
-  void setPreviousViewType(StringHandler::ViewType viewType);
-  StringHandler::ViewType getPreviousViewType();
-  void setShowGridLines(bool On);
-  bool isShowGridLines();
+  void setPreviousViewType(StringHandler::ViewType viewType) {mPreviousViewType = viewType;}
+  StringHandler::ViewType getPreviousViewType() {return mPreviousViewType;}
+  void setShowGridLines(bool On) {mShowGridLines = On;}
+  bool isShowGridLines() {return mShowGridLines;}
   bool eventFilter(QObject *object, QEvent *event);
   void changeRecentModelsListSelection(bool moveDown);
 private:
@@ -445,12 +464,14 @@ private:
   QListWidget *mpRecentModelsList;
   void loadPreviousViewType(ModelWidget *pModelWidget);
 public slots:
-  void openRecentModelWidget(QListWidgetItem *pItem);
+  bool openRecentModelWidget(QListWidgetItem *pListWidgetItem);
   void currentModelWidgetChanged(QMdiSubWindow *pSubWindow);
   void saveModelWidget();
   void saveAsModelWidget();
   void saveTotalModelWidget();
   void printModel();
+  void showSimulationParams();
+  void alignInterfaces();
 };
 
 #endif // MODELWIDGETCONTAINER_H
