@@ -29,25 +29,31 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- * RCS: $Id$
- *
  */
 
 #ifndef OPTIONSDIALOG_H
 #define OPTIONSDIALOG_H
 
-#include "MainWindow.h"
-#include "Helper.h"
-#include "Utilities.h"
+#include "Util/Helper.h"
+#include "Util/Utilities.h"
+
+#include <QFontComboBox>
+#include <QStackedWidget>
+#include <QDialogButtonBox>
+#include <QRadioButton>
+#include <QTreeWidget>
+#include <QDialog>
+#include <QLineEdit>
 
-class MainWindow;
 class GeneralSettingsPage;
 class LibrariesPage;
-class ModelicaTextHighlighter;
-class ModelicaTextEditorPage;
+class TextEditorPage;
+class ModelicaEditorPage;
+class MetaModelicaEditorPage;
+class MetaModelEditorPage;
+class CEditorPage;
+class HTMLEditorPage;
 class GraphicalViewsPage;
 class SimulationPage;
 class MessagesPage;
@@ -59,18 +65,32 @@ class FigaroPage;
 class DebuggerPage;
 class FMIPage;
 class TLMPage;
-class TLMEditorPage;
 class TabSettings;
+class StackFramesWidget;
 
 class OptionsDialog : public QDialog
 {
   Q_OBJECT
+private:
+  // the only class that is allowed to create and destroy
+  friend class MainWindow;
+
+  static void create();
+  static void destroy();
+  OptionsDialog(QWidget *pParent = 0);
+
+  static OptionsDialog *mpInstance;
 public:
-  OptionsDialog(MainWindow *pMainWindow);
+  static OptionsDialog* instance() {return mpInstance;}
   void readSettings();
   void readGeneralSettings();
   void readLibrariesSettings();
-  void readModelicaTextSettings();
+  void readTextEditorSettings();
+  void readModelicaEditorSettings();
+  void readMetaModelicaEditorSettings();
+  void readMetaModelEditorSettings();
+  void readCEditorSettings();
+  void readHTMLEditorSettings();
   void readGraphicalViewsSettings();
   void readSimulationSettings();
   void readMessagesSettings();
@@ -82,12 +102,15 @@ public:
   void readDebuggerSettings();
   void readFMISettings();
   void readTLMSettings();
-  void readTLMEditorSettings();
   void saveGeneralSettings();
   void saveLibrariesSettings();
-  void saveModelicaTextSettings();
+  void saveTextEditorSettings();
+  void saveModelicaEditorSettings();
+  void saveMetaModelicaEditorSettings();
+  void saveMetaModelEditorSettings();
+  void saveCEditorSettings();
+  void saveHTMLEditorSettings();
   void saveTLMSettings();
-  void saveTLMEditorSettings();
   void saveGraphicalViewsSettings();
   void saveSimulationSettings();
   void saveMessagesSettings();
@@ -101,9 +124,14 @@ public:
   void setUpDialog();
   void addListItems();
   void createPages();
-  MainWindow* getMainWindow() {return mpMainWindow;}
   GeneralSettingsPage* getGeneralSettingsPage() {return mpGeneralSettingsPage;}
-  ModelicaTextEditorPage* getModelicaTextEditorPage() {return mpModelicaTextEditorPage;}
+  LibrariesPage* getLibrariesPage() {return mpLibrariesPage;}
+  TextEditorPage* getTextEditorPage() {return mpTextEditorPage;}
+  ModelicaEditorPage* getModelicaEditorPage() {return mpModelicaEditorPage;}
+  MetaModelicaEditorPage* getMetaModelicaEditorPage() {return mpMetaModelicaEditorPage;}
+  MetaModelEditorPage* getMetaModelEditorPage() {return mpMetaModelEditorPage;}
+  CEditorPage* getCEditorPage() {return mpCEditorPage;}
+  HTMLEditorPage* getHTMLEditorPage() {return mpHTMLEditorPage;}
   GraphicalViewsPage* getGraphicalViewsPage() {return mpGraphicalViewsPage;}
   SimulationPage* getSimulationPage() {return mpSimulationPage;}
   MessagesPage* getMessagesPage() {return mpMessagesPage;}
@@ -115,24 +143,29 @@ public:
   DebuggerPage* getDebuggerPage() {return mpDebuggerPage;}
   FMIPage* getFMIPage() {return mpFMIPage;}
   TLMPage* getTLMPage() {return mpTLMPage;}
-  TLMEditorPage* getTLMEditorPage() {return mpTLMEditorPage;}
   void saveDialogGeometry();
   void show();
-  TabSettings getModelicaTabSettings();
-  TabSettings getTLMTabSettings();
+  TabSettings getTabSettings();
 signals:
-  void modelicaTextSettingsChanged();
-  void TLMEditorSettingsChanged();
-  void updateLineWrapping();
+  void textSettingsChanged();
+  void modelicaEditorSettingsChanged();
+  void metaModelicaEditorSettingsChanged();
+  void metaModelEditorSettingsChanged();
+  void cEditorSettingsChanged();
+  void HTMLEditorSettingsChanged();
 public slots:
   void changePage(QListWidgetItem *current, QListWidgetItem *previous);
   void reject();
   void saveSettings();
 private:
-  MainWindow *mpMainWindow;
   GeneralSettingsPage *mpGeneralSettingsPage;
   LibrariesPage *mpLibrariesPage;
-  ModelicaTextEditorPage *mpModelicaTextEditorPage;
+  TextEditorPage *mpTextEditorPage;
+  ModelicaEditorPage *mpModelicaEditorPage;
+  MetaModelicaEditorPage *mpMetaModelicaEditorPage;
+  MetaModelEditorPage *mpMetaModelEditorPage;
+  CEditorPage *mpCEditorPage;
+  HTMLEditorPage *mpHTMLEditorPage;
   GraphicalViewsPage *mpGraphicalViewsPage;
   SimulationPage *mpSimulationPage;
   MessagesPage *mpMessagesPage;
@@ -144,7 +177,6 @@ private:
   DebuggerPage *mpDebuggerPage;
   FMIPage *mpFMIPage;
   TLMPage *mpTLMPage;
-  TLMEditorPage *mpTLMEditorPage;
   QSettings *mpSettings;
   QListWidget *mpOptionsList;
   QStackedWidget *mpPagesWidget;
@@ -169,6 +201,7 @@ public:
   QString getTerminalCommand() {return mpTerminalCommandTextBox->text();}
   void setTerminalCommandArguments(QString value) {mpTerminalCommandArgumentsTextBox->setText(value);}
   QString getTerminalCommandArguments() {return mpTerminalCommandArgumentsTextBox->text();}
+  QCheckBox* getHideVariablesBrowserCheckBox() {return mpHideVariablesBrowserCheckBox;}
   QSpinBox* getLibraryIconSizeSpinBox() {return mpLibraryIconSizeSpinBox;}
   void setShowProtectedClasses(bool value);
   bool getShowProtectedClasses();
@@ -176,13 +209,11 @@ public:
   QString getModelingViewMode();
   void setDefaultView(QString value);
   QString getDefaultView();
-  QGroupBox* getEnableAutoSaveGroupBox();
-  QSpinBox* getAutoSaveIntervalSpinBox();
-  QCheckBox* getEnableAutoSaveForSingleClassesCheckBox();
-  QCheckBox* getEnableAutoSaveForOneFilePackagesCheckBox();
+  QGroupBox* getEnableAutoSaveGroupBox() {return mpEnableAutoSaveGroupBox;}
+  QSpinBox* getAutoSaveIntervalSpinBox() {return mpAutoSaveIntervalSpinBox;}
   int getWelcomePageView();
   void setWelcomePageView(int view);
-  QCheckBox* getShowLatestNewsCheckBox();
+  QCheckBox* getShowLatestNewsCheckBox() {return mpShowLatestNewsCheckBox;}
 private:
   OptionsDialog *mpOptionsDialog;
   QGroupBox *mpGeneralSettingsGroupBox;
@@ -199,6 +230,7 @@ private:
   QPushButton *mpTerminalCommandBrowseButton;
   Label *mpTerminalCommandArgumentsLabel;
   QLineEdit *mpTerminalCommandArgumentsTextBox;
+  QCheckBox *mpHideVariablesBrowserCheckBox;
   QGroupBox *mpLibrariesBrowserGroupBox;
   Label *mpLibraryIconSizeLabel;
   QSpinBox *mpLibraryIconSizeSpinBox;
@@ -215,8 +247,6 @@ private:
   Label *mpAutoSaveIntervalLabel;
   QSpinBox *mpAutoSaveIntervalSpinBox;
   Label *mpAutoSaveSecondsLabel;
-  QCheckBox *mpEnableAutoSaveForSingleClassesCheckBox;
-  QCheckBox *mpEnableAutoSaveForOneFilePackagesCheckBox;
   QGroupBox *mpWelcomePageGroupBox;
   QRadioButton *mpHorizontalViewRadioButton;
   QRadioButton *mpVerticalViewRadioButton;
@@ -232,9 +262,10 @@ class LibrariesPage : public QWidget
   Q_OBJECT
 public:
   LibrariesPage(OptionsDialog *pOptionsDialog);
-  QTreeWidget* getSystemLibrariesTree();
-  QCheckBox* getForceModelicaLoadCheckBox();
-  QTreeWidget* getUserLibrariesTree();
+  QTreeWidget* getSystemLibrariesTree() {return mpSystemLibrariesTree;}
+  QCheckBox* getForceModelicaLoadCheckBox() {return mpForceModelicaLoadCheckBox;}
+  QCheckBox* getLoadOpenModelicaLibraryCheckBox() {return mpLoadOpenModelicaOnStartupCheckBox;}
+  QTreeWidget* getUserLibrariesTree() {return mpUserLibrariesTree;}
   OptionsDialog *mpOptionsDialog;
 private:
   QGroupBox *mpSystemLibrariesGroupBox;
@@ -245,6 +276,7 @@ private:
   QPushButton *mpEditSystemLibraryButton;
   QDialogButtonBox *mpSystemLibrariesButtonBox;
   QCheckBox *mpForceModelicaLoadCheckBox;
+  QCheckBox *mpLoadOpenModelicaOnStartupCheckBox;
   QGroupBox *mpUserLibrariesGroupBox;
   QTreeWidget *mpUserLibrariesTree;
   QPushButton *mpAddUserLibraryButton;
@@ -299,36 +331,29 @@ private slots:
   void addUserLibrary();
 };
 
-class ModelicaTextEditorPage : public QWidget
+class TextEditorPage : public QWidget
 {
   Q_OBJECT
 public:
-  ModelicaTextEditorPage(OptionsDialog *pOptionsDialog);
+  TextEditorPage(OptionsDialog *pOptionsDialog);
+  QComboBox *getLineEndingComboBox() {return mpLineEndingComboBox;}
+  QComboBox *getBOMComboBox() {return mpBOMComboBox;}
   QComboBox *getTabPolicyComboBox() {return mpTabPolicyComboBox;}
   QSpinBox *getTabSizeSpinBox() {return mpTabSizeSpinBox;}
   QSpinBox *getIndentSpinBox() {return mpIndentSpinBox;}
-  QCheckBox* getSyntaxHighlightingCheckbox() {return mpSyntaxHighlightingCheckbox;}
+  QGroupBox* getSyntaxHighlightingGroupBox() {return mpSyntaxHighlightingGroupBox;}
+  QCheckBox* getCodeFoldingCheckBox() {return mpCodeFoldingCheckBox;}
   QCheckBox* getMatchParenthesesCommentsQuotesCheckBox() {return mpMatchParenthesesCommentsQuotesCheckBox;}
   QCheckBox* getLineWrappingCheckbox() {return mpLineWrappingCheckbox;}
   QFontComboBox* getFontFamilyComboBox() {return mpFontFamilyComboBox;}
   DoubleSpinBox* getFontSizeSpinBox() {return mpFontSizeSpinBox;}
-  void addListItems();
-  void setTextRuleColor(QColor color);
-  QColor getTextRuleColor() {return mTextColor;}
-  void setNumberRuleColor(QColor color);
-  QColor getNumberRuleColor() {return mNumberColor;}
-  void setKeywordRuleColor(QColor color);
-  QColor getKeywordRuleColor() {return mKeywordColor;}
-  void setTypeRuleColor(QColor color);
-  QColor getTypeRuleColor() {return mTypeColor;}
-  void setFunctionRuleColor(QColor color);
-  QColor getFunctionRuleColor() {return mFunctionColor;}
-  void setQuotesRuleColor(QColor color);
-  QColor getQuotesRuleColor() {return mQuotesColor;}
-  void setCommentRuleColor(QColor color);
-  QColor getCommentRuleColor() {return mCommentColor;}
 private:
   OptionsDialog *mpOptionsDialog;
+  QGroupBox *mpFormatGroupBox;
+  Label *mpLineEndingLabel;
+  QComboBox *mpLineEndingComboBox;
+  Label *mpBOMLabel;
+  QComboBox *mpBOMComboBox;
   QGroupBox *mpTabsAndIndentation;
   Label *mpTabPolicyLabel;
   QComboBox *mpTabPolicyComboBox;
@@ -337,39 +362,107 @@ private:
   Label *mpIndentSizeLabel;
   QSpinBox *mpIndentSpinBox;
   QGroupBox *mpSyntaxHighlightAndTextWrappingGroupBox;
-  QCheckBox *mpSyntaxHighlightingCheckbox;
+  QGroupBox *mpSyntaxHighlightingGroupBox;
+  QCheckBox *mpCodeFoldingCheckBox;
   QCheckBox *mpMatchParenthesesCommentsQuotesCheckBox;
   QCheckBox *mpLineWrappingCheckbox;
-  QGroupBox *mpFontColorsGroupBox;
+  QGroupBox *mpFontGroupBox;
   Label *mpFontFamilyLabel;
   QFontComboBox *mpFontFamilyComboBox;
   Label *mpFontSizeLabel;
   DoubleSpinBox *mpFontSizeSpinBox;
-  Label *mpItemsLabel;
-  QListWidget *mpItemsList;
-  Label *mpItemColorLabel;
-  QPushButton *mpItemColorPickButton;
-  Label *mpPreviewLabel;
-  QPlainTextEdit *mpPreviewPlainTextBox;
-  QColor mTextColor;
-  QListWidgetItem *mpTextItem;
-  QColor mNumberColor;
-  QListWidgetItem *mpNumberItem;
-  QColor mKeywordColor;
-  QListWidgetItem *mpKeywordItem;
-  QColor mTypeColor;
-  QListWidgetItem *mpTypeItem;
-  QColor mFunctionColor;
-  QListWidgetItem *mpFunctionItem;
-  QColor mQuotesColor;
-  QListWidgetItem *mpQuotesItem;
-  QColor mCommentColor;
-  QListWidgetItem *mpCommentItem;
+};
+
+class ModelicaEditorPage : public QWidget
+{
+  Q_OBJECT
+public:
+  ModelicaEditorPage(OptionsDialog *pOptionsDialog);
+  OptionsDialog* getOptionsDialog() {return mpOptionsDialog;}
+  QCheckBox *getPreserveTextIndentationCheckBox() {return mpPreserveTextIndentationCheckBox;}
+  void setColor(QString item, QColor color);
+  QColor getColor(QString item);
+  void emitUpdatePreview() {emit updatePreview();}
+private:
+  OptionsDialog *mpOptionsDialog;
+  QCheckBox *mpPreserveTextIndentationCheckBox;
+  CodeColorsWidget *mpCodeColorsWidget;
 signals:
   void updatePreview();
 public slots:
-  void setLineWrapping();
-  void pickColor();
+  void setLineWrapping(bool enabled);
+};
+
+class MetaModelicaEditorPage : public QWidget
+{
+  Q_OBJECT
+public:
+  MetaModelicaEditorPage(OptionsDialog *pOptionsDialog);
+  OptionsDialog* getOptionsDialog() {return mpOptionsDialog;}
+  void setColor(QString item, QColor color);
+  QColor getColor(QString item);
+  void emitUpdatePreview() {emit updatePreview();}
+private:
+  OptionsDialog *mpOptionsDialog;
+  CodeColorsWidget *mpCodeColorsWidget;
+signals:
+  void updatePreview();
+public slots:
+  void setLineWrapping(bool enabled);
+};
+
+class MetaModelEditorPage : public QWidget
+{
+  Q_OBJECT
+public:
+  MetaModelEditorPage(OptionsDialog *pOptionsDialog);
+  OptionsDialog* getOptionsDialog() {return mpOptionsDialog;}
+  void setColor(QString item, QColor color);
+  QColor getColor(QString item);
+  void emitUpdatePreview() {emit updatePreview();}
+private:
+  OptionsDialog *mpOptionsDialog;
+  CodeColorsWidget *mpCodeColorsWidget;
+signals:
+  void updatePreview();
+public slots:
+  void setLineWrapping(bool enabled);
+};
+
+class CEditorPage : public QWidget
+{
+  Q_OBJECT
+public:
+  CEditorPage(OptionsDialog *pOptionsDialog);
+  OptionsDialog* getOptionsDialog() {return mpOptionsDialog;}
+  void setColor(QString item, QColor color);
+  QColor getColor(QString item);
+  void emitUpdatePreview() {emit updatePreview();}
+private:
+  OptionsDialog *mpOptionsDialog;
+  CodeColorsWidget *mpCodeColorsWidget;
+signals:
+  void updatePreview();
+public slots:
+  void setLineWrapping(bool enabled);
+};
+
+class HTMLEditorPage : public QWidget
+{
+  Q_OBJECT
+public:
+  HTMLEditorPage(OptionsDialog *pOptionsDialog);
+  OptionsDialog* getOptionsDialog() {return mpOptionsDialog;}
+  void setColor(QString item, QColor color);
+  QColor getColor(QString item);
+  void emitUpdatePreview() {emit updatePreview();}
+private:
+  OptionsDialog *mpOptionsDialog;
+  CodeColorsWidget *mpCodeColorsWidget;
+signals:
+  void updatePreview();
+public slots:
+  void setLineWrapping(bool enabled);
 };
 
 class GraphicalViewsPage : public QWidget
@@ -462,7 +555,10 @@ public:
   QComboBox* getTargetLanguageComboBox() {return mpTargetLanguageComboBox;}
   QComboBox* getTargetCompilerComboBox() {return mpTargetCompilerComboBox;}
   QLineEdit* getOMCFlagsTextBox() {return mpOMCFlagsTextBox;}
-  QCheckBox *getSaveClassBeforeSimulationCheckBox() {return mpSaveClassBeforeSimulationCheckBox;}
+  QCheckBox* getIgnoreCommandLineOptionsAnnotationCheckBox() {return mpIgnoreCommandLineOptionsAnnotationCheckBox;}
+  QCheckBox* getIgnoreSimulationFlagsAnnotationCheckBox() {return mpIgnoreSimulationFlagsAnnotationCheckBox;}
+  QCheckBox* getSaveClassBeforeSimulationCheckBox() {return mpSaveClassBeforeSimulationCheckBox;}
+  QCheckBox* getSwitchToPlottingPerspectiveCheckBox() {return mpSwitchToPlottingPerspectiveCheckBox;}
   void setOutputMode(QString value);
   QString getOutputMode();
 private:
@@ -478,10 +574,18 @@ private:
   QComboBox *mpTargetCompilerComboBox;
   Label *mpOMCFlagsLabel;
   QLineEdit *mpOMCFlagsTextBox;
+  QToolButton *mpOMCFlagsHelpButton;
+  QCheckBox *mpIgnoreCommandLineOptionsAnnotationCheckBox;
+  QCheckBox *mpIgnoreSimulationFlagsAnnotationCheckBox;
   QCheckBox *mpSaveClassBeforeSimulationCheckBox;
+  QCheckBox *mpSwitchToPlottingPerspectiveCheckBox;
   QGroupBox *mpOutputGroupBox;
   QRadioButton *mpStructuredRadioButton;
   QRadioButton *mpFormattedTextRadioButton;
+public slots:
+  void updateMatchingAlgorithmToolTip(int index);
+  void updateIndexReductionToolTip(int index);
+  void showOMCFlagsHelp();
 };
 
 class MessagesPage : public QWidget
@@ -533,11 +637,12 @@ class NotificationsPage : public QWidget
   Q_OBJECT
 public:
   NotificationsPage(OptionsDialog *pOptionsDialog);
-  QCheckBox* getQuitApplicationCheckBox();
-  QCheckBox* getItemDroppedOnItselfCheckBox();
-  QCheckBox* getReplaceableIfPartialCheckBox();
-  QCheckBox* getInnerModelNameChangedCheckBox();
-  QCheckBox* getSaveModelForBitmapInsertionCheckBox();
+  QCheckBox* getQuitApplicationCheckBox() {return mpQuitApplicationCheckBox;}
+  QCheckBox* getItemDroppedOnItselfCheckBox() {return mpItemDroppedOnItselfCheckBox;}
+  QCheckBox* getReplaceableIfPartialCheckBox() {return mpReplaceableIfPartialCheckBox;}
+  QCheckBox* getInnerModelNameChangedCheckBox() {return mpInnerModelNameChangedCheckBox;}
+  QCheckBox* getSaveModelForBitmapInsertionCheckBox() {return mpSaveModelForBitmapInsertionCheckBox;}
+  QCheckBox* getAlwaysAskForDraggedComponentName() {return mpAlwaysAskForDraggedComponentName;}
 private:
   OptionsDialog *mpOptionsDialog;
   QGroupBox *mpNotificationsGroupBox;
@@ -546,6 +651,7 @@ private:
   QCheckBox *mpReplaceableIfPartialCheckBox;
   QCheckBox *mpInnerModelNameChangedCheckBox;
   QCheckBox *mpSaveModelForBitmapInsertionCheckBox;
+  QCheckBox *mpAlwaysAskForDraggedComponentName;
 };
 
 class LineStylePage : public QWidget
@@ -656,12 +762,15 @@ private:
   QLineEdit *mpFigaroOptionsFileTextBox;
   QPushButton *mpBrowseFigaroOptionsFileButton;
   Label *mpFigaroProcessLabel;
+  QString mFigaroProcessPath;
   QLineEdit *mpFigaroProcessTextBox;
   QPushButton *mpBrowseFigaroProcessButton;
+  QPushButton *mpResetFigaroProcessButton;
 private slots:
   void browseFigaroLibraryFile();
   void browseFigaroOptionsFile();
   void browseFigaroProcessFile();
+  void resetFigaroProcessPath();
 };
 
 class DebuggerPage : public QWidget
@@ -671,6 +780,7 @@ public:
   DebuggerPage(OptionsDialog *pOptionsDialog);
   void setGDBPath(QString path);
   QString getGDBPath();
+  QString getGDBPathForSettings() {return mpGDBPathTextBox->text();}
   QSpinBox* getGDBCommandTimeoutSpinBox() {return mpGDBCommandTimeoutSpinBox;}
   QSpinBox* getGDBOutputLimitSpinBox() {return mpGDBOutputLimitSpinBox;}
   QCheckBox* getDisplayCFramesCheckBox() {return mpDisplayCFramesCheckBox;}
@@ -710,6 +820,8 @@ public:
   void setFMIExportType(QString type);
   QString getFMIExportType();
   QLineEdit* getFMUNameTextBox() {return mpFMUNameTextBox;}
+  QGroupBox* getPlatformsGroupBox() {return mpPlatformsGroupBox;}
+  QComboBox* getLinkingComboBox() {return mpLinkingComboBox;}
 private:
   OptionsDialog *mpOptionsDialog;
   QGroupBox *mpExportGroupBox;
@@ -722,6 +834,8 @@ private:
   QRadioButton *mpModelExchangeCoSimulationRadioButton;
   Label *mpFMUNameLabel;
   QLineEdit *mpFMUNameTextBox;
+  QGroupBox *mpPlatformsGroupBox;
+  QComboBox *mpLinkingComboBox;
 };
 
 class TLMPage : public QWidget
@@ -750,67 +864,4 @@ private slots:
   void browseTLMMonitorProcess();
 };
 
-class TLMEditorPage : public QWidget
-{
-  Q_OBJECT
-public:
-  TLMEditorPage(OptionsDialog *pOptionsDialog);
-  QComboBox *getTabPolicyComboBox() {return mpTabPolicyComboBox;}
-  QSpinBox *getTabSizeSpinBox() {return mpTabSizeSpinBox;}
-  QSpinBox *getIndentSpinBox() {return mpIndentSpinBox;}
-  QCheckBox* getSyntaxHighlightingCheckbox() {return mpSyntaxHighlightingCheckbox;}
-  QCheckBox* getLineWrappingCheckbox() {return mpLineWrappingCheckbox;}
-  QFontComboBox* getFontFamilyComboBox() {return mpFontFamilyComboBox;}
-  DoubleSpinBox* getFontSizeSpinBox() {return mpFontSizeSpinBox;}
-  void addListItems();
-  void setTextRuleColor(QColor color);
-  QColor getTextRuleColor(){return mTextColor;}
-  void setQuotesRuleColor(QColor color);
-  QColor getQuotesRuleColor(){return mQuotesColor;}
-  void setCommentRuleColor(QColor color);
-  QColor getCommentRuleColor(){return mCommentColor;}
-  void setTagRuleColor(QColor color);
-  QColor getTagRuleColor(){return mTagColor;}
-  void setElementRuleColor(QColor color);
-  QColor getElementRuleColor(){return mElementColor;}
-private:
-  OptionsDialog *mpOptionsDialog;
-  QGroupBox *mpTabsAndIndentation;
-  Label *mpTabPolicyLabel;
-  QComboBox *mpTabPolicyComboBox;
-  Label *mpTabSizeLabel;
-  QSpinBox *mpTabSizeSpinBox;
-  Label *mpIndentSizeLabel;
-  QSpinBox *mpIndentSpinBox;
-  QGroupBox *mpSyntaxHighlightAndTextWrappingGroupBox;
-  QCheckBox *mpSyntaxHighlightingCheckbox;
-  QCheckBox *mpLineWrappingCheckbox;
-  QGroupBox *mpFontColorsGroupBox;
-  Label *mpFontFamilyLabel;
-  QFontComboBox *mpFontFamilyComboBox;
-  Label *mpFontSizeLabel;
-  DoubleSpinBox *mpFontSizeSpinBox;
-  Label *mpItemsLabel;
-  QListWidget *mpItemsList;
-  Label *mpItemColorLabel;
-  QPushButton *mpItemColorPickButton;
-  Label *mpPreviewLabel;
-  QPlainTextEdit *mpPreviewPlainTextBox;
-  QColor mTextColor;
-  QListWidgetItem *mpTextItem;
-  QColor mQuotesColor;
-  QListWidgetItem *mpQuotesItem;
-  QColor mCommentColor;
-  QListWidgetItem *mpCommentItem;
-  QColor mTagColor;
-  QListWidgetItem *mpTagItem;
-  QColor mElementColor;
-  QListWidgetItem *mpElementItem;
-signals:
-  void updatePreview();
-public slots:
-  void setLineWrapping();
-  void pickColor();
-};
-
 #endif // OPTIONSDIALOG_H
