@@ -29,16 +29,22 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- * RCS: $Id$
- *
  */
 
 #include "TransformationsWidget.h"
+#include "Options/OptionsDialog.h"
+#include "Util/StringHandler.h"
+#include "Modeling/LibraryTreeWidget.h"
+#include "Editors/TransformationsEditor.h"
+#include "Editors/ModelicaEditor.h"
 #include <qjson/parser.h>
 
+#include <QStatusBar>
+#include <QGridLayout>
+#include <QVBoxLayout>
+#include <QMessageBox>
+
 /*!
   \class TVariablesTreeItem
   \brief Contains the information about the result variable.
@@ -387,7 +393,7 @@ bool TVariableTreeProxyModel::filterAcceptsRow(int sourceRow, const QModelIndex
       if (pTVariablesTreeItem)
       {
         QString variableName = pTVariablesTreeItem->getVariableName();
-        variableName.remove(QRegExp("(_res.mat|_res.plt|_res.csv)"));
+        variableName.remove(QRegExp("(\\.mat|\\.plt|\\.csv|_res.mat|_res.plt|_res.csv)"));
         return variableName.contains(filterRegExp());
       }
       else
@@ -405,13 +411,13 @@ TVariablesTreeView::TVariablesTreeView(TransformationsWidget *pTransformationsWi
   : QTreeView(pTransformationsWidget)
 {
   mpTransformationsWidget = pTransformationsWidget;
-  setObjectName("TreeWithBranches");
   setItemDelegate(new ItemDelegate(this));
   setTextElideMode(Qt::ElideMiddle);
   setIndentation(Helper::treeIndentation);
   setSortingEnabled(true);
   sortByColumn(0, Qt::AscendingOrder);
   setExpandsOnDoubleClick(false);
+  setUniformRowHeights(true);
 }
 
 EquationTreeWidget::EquationTreeWidget(TransformationsWidget *pTransformationWidget)
@@ -437,17 +443,17 @@ EquationTreeWidget::EquationTreeWidget(TransformationsWidget *pTransformationWid
   connect(this, SIGNAL(itemDoubleClicked(QTreeWidgetItem*,int)), mpTransformationWidget, SLOT(fetchEquationData(QTreeWidgetItem*,int)));
 }
 
-TransformationsWidget::TransformationsWidget(QString infoXMLFullFileName, MainWindow *pMainWindow)
-  : mpMainWindow(pMainWindow), mInfoXMLFullFileName(infoXMLFullFileName)
+TransformationsWidget::TransformationsWidget(QString infoJSONFullFileName, QWidget *pParent)
+  : QWidget(pParent), mInfoJSONFullFileName(infoJSONFullFileName)
 {
-  if (!mInfoXMLFullFileName.endsWith("_info.json")) {
+  if (!mInfoJSONFullFileName.endsWith("_info.json")) {
     mProfJSONFullFileName = "";
     mProfilingDataRealFileName = "";
   } else {
-    mProfJSONFullFileName = infoXMLFullFileName.left(infoXMLFullFileName.size() - 9) + "prof.json";
-    mProfilingDataRealFileName = infoXMLFullFileName.left(infoXMLFullFileName.size() - 9) + "prof.realdata";
+    mProfJSONFullFileName = infoJSONFullFileName.left(infoJSONFullFileName.size() - 9) + "prof.json";
+    mProfilingDataRealFileName = infoJSONFullFileName.left(infoJSONFullFileName.size() - 9) + "prof.realdata";
   }
-  setWindowIcon(QIcon(":/Resources/icons/debugger.svg"));
+  setWindowIcon(QIcon(":/Resources/icons/equational-debugger.svg"));
   setWindowTitle(QString(Helper::applicationName).append(" - ").append(Helper::transformationalDebugger));
   QToolButton *pReloadToolButton = new QToolButton;
   pReloadToolButton->setToolTip(Helper::reload);
@@ -455,7 +461,7 @@ TransformationsWidget::TransformationsWidget(QString infoXMLFullFileName, MainWi
   pReloadToolButton->setIcon(QIcon(":/Resources/icons/refresh.svg"));
   connect(pReloadToolButton, SIGNAL(clicked()), SLOT(reloadTransformations()));
   /* info xml file path label */
-  Label *pInfoXMLFilePathLabel = new Label(mInfoXMLFullFileName, this);
+  Label *pInfoXMLFilePathLabel = new Label(mInfoJSONFullFileName, this);
   pInfoXMLFilePathLabel->setElideMode(Qt::ElideMiddle);
   /* create status bar */
   QStatusBar *pStatusBar = new QStatusBar;
@@ -468,9 +474,9 @@ TransformationsWidget::TransformationsWidget(QString infoXMLFullFileName, MainWi
   pVariablesBrowserLabel->setObjectName("LabelWithBorder");
   // tree search filters
   mpTreeSearchFilters = new TreeSearchFilters(this);
-  mpTreeSearchFilters->getSearchTextBox()->setPlaceholderText(Helper::findVariables);
-  connect(mpTreeSearchFilters->getSearchTextBox(), SIGNAL(returnPressed()), SLOT(findVariables()));
-  connect(mpTreeSearchFilters->getSearchTextBox(), SIGNAL(textEdited(QString)), SLOT(findVariables()));
+  mpTreeSearchFilters->getFilterTextBox()->setPlaceholderText(Helper::filterVariables);
+  connect(mpTreeSearchFilters->getFilterTextBox(), SIGNAL(returnPressed()), SLOT(findVariables()));
+  connect(mpTreeSearchFilters->getFilterTextBox(), SIGNAL(textEdited(QString)), SLOT(findVariables()));
   connect(mpTreeSearchFilters->getCaseSensitiveCheckBox(), SIGNAL(toggled(bool)), SLOT(findVariables()));
   connect(mpTreeSearchFilters->getSyntaxComboBox(), SIGNAL(currentIndexChanged(int)), SLOT(findVariables()));
   /* variables tree view */
@@ -535,7 +541,6 @@ TransformationsWidget::TransformationsWidget(QString infoXMLFullFileName, MainWi
   pEquationsBrowserLabel->setObjectName("LabelWithBorder");
   /* Equations tree widget */
   mpEquationsTreeWidget = new EquationTreeWidget(this);
-  mpEquationsTreeWidget->setObjectName("TreeWithBranches");
   mpEquationsTreeWidget->setIndentation(Helper::treeIndentation);
   QGridLayout *pEquationsGridLayout = new QGridLayout;
   pEquationsGridLayout->setSpacing(1);
@@ -607,10 +612,10 @@ TransformationsWidget::TransformationsWidget(QString infoXMLFullFileName, MainWi
   mpTSourceEditorInfoBar = new InfoBar(this);
   mpTSourceEditorInfoBar->hide();
   mpTransformationsEditor = new TransformationsEditor(this);
-  ModelicaTextHighlighter *pModelicaTextHighlighter;
-  pModelicaTextHighlighter = new ModelicaTextHighlighter(mpMainWindow->getOptionsDialog()->getModelicaTextEditorPage(),
-                                                         mpTransformationsEditor->getPlainTextEdit());
-  connect(mpMainWindow->getOptionsDialog(), SIGNAL(modelicaTextSettingsChanged()), pModelicaTextHighlighter, SLOT(settingsChanged()));
+  ModelicaHighlighter *pModelicaTextHighlighter;
+  pModelicaTextHighlighter = new ModelicaHighlighter(OptionsDialog::instance()->getModelicaEditorPage(),
+                                                     mpTransformationsEditor->getPlainTextEdit());
+  connect(OptionsDialog::instance(), SIGNAL(modelicaEditorSettingsChanged()), pModelicaTextHighlighter, SLOT(settingsChanged()));
   QVBoxLayout *pTSourceEditorVerticalLayout = new QVBoxLayout;
   pTSourceEditorVerticalLayout->setSpacing(1);
   pTSourceEditorVerticalLayout->setContentsMargins(0, 0, 0, 0);
@@ -716,8 +721,8 @@ TransformationsWidget::TransformationsWidget(QString infoXMLFullFileName, MainWi
   pMainLayout->addWidget(mpTransformationsHorizontalSplitter, 1, 0);
   setLayout(pMainLayout);
   /* restore the TransformationsWidget geometry and splitters state. */
-  QSettings *pSettings = OpenModelica::getApplicationSettings();
-  if (mpMainWindow->getOptionsDialog()->getGeneralSettingsPage()->getPreserveUserCustomizations())
+  QSettings *pSettings = Utilities::getApplicationSettings();
+  if (OptionsDialog::instance()->getGeneralSettingsPage()->getPreserveUserCustomizations())
   {
     pSettings->beginGroup("transformationalDebugger");
     restoreGeometry(pSettings->value("geometry").toByteArray());
@@ -767,6 +772,7 @@ static OMOperation* variantToOperationPtr(QVariantMap var)
 
 static void variantToSource(QVariantMap var, OMInfo &info, QStringList &types, QList<OMOperation*> &ops)
 {
+  Q_UNUSED(types);
   QVariantMap vinfo = var["info"].toMap();
   info.file = vinfo["file"].toString();
   info.lineStart = vinfo["lineStart"].toInt();
@@ -795,17 +801,17 @@ static OMEquation* getOMEquation(QList<OMEquation*> equations, int index)
 
 void TransformationsWidget::loadTransformations()
 {
-  QFile file(mInfoXMLFullFileName);
+  QFile file(mInfoJSONFullFileName);
   mEquations.clear();
   mVariables.clear();
   hasOperationsEnabled = false;
-  if (mInfoXMLFullFileName.endsWith(".json")) {
+  if (mInfoJSONFullFileName.endsWith(".json")) {
     QJson::Parser parser;
     bool ok;
     QVariantMap result;
     result = parser.parse(&file, &ok).toMap();
     if (!ok) {
-      QMessageBox::critical(this, QString(Helper::applicationName).append(" - ").append(Helper::parsingFailedJson), Helper::parsingFailedJson + ": " + mInfoXMLFullFileName, Helper::ok);
+      QMessageBox::critical(this, QString(Helper::applicationName).append(" - ").append(Helper::parsingFailedJson), Helper::parsingFailedJson + ": " + mInfoJSONFullFileName, Helper::ok);
       return;
     }
     QVariantMap vars = result["variables"].toMap();
@@ -1162,9 +1168,9 @@ void TransformationsWidget::reloadTransformations()
   /* Clear the variable operations tree. */
   clearTreeWidgetItems(mpVariableOperationsTreeWidget);
   /* clear the variable tree filters. */
-  bool signalsState = mpTreeSearchFilters->getSearchTextBox()->blockSignals(true);
-  mpTreeSearchFilters->getSearchTextBox()->clear();
-  mpTreeSearchFilters->getSearchTextBox()->blockSignals(signalsState);
+  bool signalsState = mpTreeSearchFilters->getFilterTextBox()->blockSignals(true);
+  mpTreeSearchFilters->getFilterTextBox()->clear();
+  mpTreeSearchFilters->getFilterTextBox()->blockSignals(signalsState);
   signalsState = mpTreeSearchFilters->getSyntaxComboBox()->blockSignals(true);
   mpTreeSearchFilters->getSyntaxComboBox()->setCurrentIndex(0);
   mpTreeSearchFilters->getSyntaxComboBox()->blockSignals(signalsState);
@@ -1197,7 +1203,7 @@ void TransformationsWidget::reloadTransformations()
  */
 void TransformationsWidget::findVariables()
 {
-  QString findText = mpTreeSearchFilters->getSearchTextBox()->text();
+  QString findText = mpTreeSearchFilters->getFilterTextBox()->text();
   QRegExp::PatternSyntax syntax = QRegExp::PatternSyntax(mpTreeSearchFilters->getSyntaxComboBox()->itemData(mpTreeSearchFilters->getSyntaxComboBox()->currentIndex()).toInt());
   Qt::CaseSensitivity caseSensitivity = mpTreeSearchFilters->getCaseSensitiveCheckBox()->isChecked() ? Qt::CaseSensitive: Qt::CaseInsensitive;
   QRegExp regExp(findText, caseSensitivity, syntax);
